<?php
/**
 * Advanced Settings Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

use Transposh_Language_Toolbox\Base_Utilities;
use Transposh_Language_Toolbox\Custom_Languages;

/**
 * Advanced settings.
 */
class Advanced_Settings extends Base_Utilities {

    /**
     * Excerpt length for alphabetic and ideographic languages.
     *
     * @var int|null
     */
    private $excerpt_length_alphabetic;
    private $excerpt_length_idiographic;

    /**
     * Initialization
     */
    public function init() {
		$this->set_settings();
		$this->set_sections();
		$this->set_fields();

        // Manage translator role
        $this->manage_translator_role();
        add_action( 'admin_init', array( $this, 'restrict_admin_access' ) );
    }

    /**
     * Load excerpt lengths from settings.
     */
    public function load_excerpts_lengths() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();

        if ( isset( $advanced_settings['activate_excerpt_length']) && $advanced_settings['activate_excerpt_length'] === '1' ) {
            $this->excerpt_length_alphabetic = isset( $advanced_settings['excerpt_length'] ) ? (int) $advanced_settings['excerpt_length'] : null;
            $this->excerpt_length_idiographic = isset( $advanced_settings['excerpt_length_special'] ) ? (int) $advanced_settings['excerpt_length_special'] : null;
            if ( ( $this->excerpt_length_alphabetic !== null ) || ( $this->excerpt_length_idiographic !== null ) ) {
                $this->apply_excerpt_filters();
            }
        }
    }

    /**
     * Apply the excerpt filters.
     */
    private function apply_excerpt_filters() {
        add_filter( 'excerpt_length', array( $this, 'tlt_excerpt_length' ), 99999 );
        add_filter( 'get_the_excerpt', array( $this, 'tlt_filter_language_excerpt' ), 99999 );
    }

    /**
     * Excerpt default length.
     *
     * @param int $length The default excerpt length.
     * @return int The default excerpt length.
     */
    public function tlt_excerpt_length( $length ) {
        $length = 500;
        return $length;
    }

    /**
     * Excerpt length for scripts that don't use spaces between words.
     *
     * @param string $excerpt The original excerpt.
     * @return string The modified excerpt.
     */
    public function tlt_filter_language_excerpt( $excerpt ) {
        // Character scripts that don't use spaces between words
        $word_based_scripts = array(
            'Han', // Chinese, Japanese, Korean
            'Thai',
            'Khmer',
            'Lao',
            'Myanmar',
            'Tibetan',
        );
        $decoration = '…';
        $limit = $this->excerpt_length_idiographic;
        $word_limit = $this->excerpt_length_alphabetic;
        if ( !$limit || !$word_limit ) {
            if ( !$limit ) {
                $limit = $word_limit * 2;
            } else {
                $word_limit = (int) $limit / 2;
            }
        }

        // Detect if the excerpt contains characters from word-based scripts
        foreach ( $word_based_scripts as $script ) {
            $pattern = "/\p{" . $script . "}+/u";
            if ( preg_match( $pattern, $excerpt ) ) {
                // If longer than limit, add decoration to the end of the string
                if ( mb_strlen( $excerpt, 'UTF-8' ) >= $limit ) {
                    return mb_substr( $excerpt, 0, $limit, 'UTF-8' ) . $decoration;
                } else {
                    return mb_substr( $excerpt, 0, $limit, 'UTF-8' );
                }
            }
        }
        
        // For other languages with spaces between words
        $words = explode( ' ', $excerpt );
        if ( count( $words ) > $word_limit ) {
            $trimmed_words = array_slice( $words, 0, $word_limit );
            $trimmed_excerpt = implode( ' ', $trimmed_words );
            return $trimmed_excerpt . $decoration;
        } else {
            return $excerpt;
        }
    }

    /**
     * Load posts exclusion if needed
     */
    public function load_posts_exclusion() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();

        if ( isset( $advanced_settings['activate_posts_exclude']) && $advanced_settings['activate_posts_exclude'] === '1' ) {
            //add_action( 'parse_query', array( $this, 'exclude_posts_by_language' ) );
            add_action( 'pre_get_posts', array( $this, 'exclude_posts_by_language' ) );
        }
    }

    /**
     * Exclude posts from the loop if they are not in the current language.
     *
     * @param WP_Query $query The WP_Query object.
     */
    public function exclude_posts_by_language( $query ) {
        if ( ! is_admin() ) { // we filter main and secondary loops (main query needs "&& $query->is_main_query()")
            // Get the current and default languages
            $current_language = $this->get_active_language();

            // Get the post types to filter
            $post_types = isset( $this->tlt_options['advanced_settings']['selected_post_types'] ) ? $this->tlt_options['advanced_settings']['selected_post_types'] : array();
            
            // Check if the "always_display_default" option is enabled
            $always_display_default = isset( $this->tlt_options['advanced_settings']['always_display_default'] ) ? ( $this->tlt_options['advanced_settings']['always_display_default'] ? true : false ) : false;

            // Check if the current post type is included in the specified array
            $current_post_type = 'post';
            if ( is_page() ) { 
                $current_post_type = 'page'; 
            }
            if ( isset( $query->query['post_type'] ) ) {
                $current_post_type = $query->query['post_type'];
            }         
            if ( in_array( $current_post_type, $post_types ) ) {
                // Modify the query to exclude posts based on language
                if ( $always_display_default ) {
                    $query->set( 'meta_query', array(
                        'relation' => 'OR',
                        array(
                            'key'     => 'tp_language',
                            'compare' => 'NOT EXISTS',
                        ),
                        array(
                            'key'     => 'tp_language',
                            'value'   => $current_language,
                            'compare' => '=',
                        ),
                    ) );
                } else {
                    if ( $current_language === $this->default_language ) {
                        $query->set( 'meta_query', array(
                            'relation' => 'OR',
                            array(
                                'key'     => 'tp_language',
                                'compare' => 'NOT EXISTS',
                            ),
                            array(
                                'key'     => 'tp_language',
                                'value'   => $this->default_language,
                                'compare' => '=',
                            ),
                        ) );
                    } else {
                        // Current language is different from the default language
                        $query->set( 'meta_query', array(
                            'relation' => 'AND',
                            array(
                                'key'     => 'tp_language',
                                'compare' => 'EXISTS',
                            ),
                            array(
                                'key'     => 'tp_language',
                                'value'   => $current_language,
                                'compare' => '=',
                            ),
                        ) );
                    }
                }
            }
        }
    }

    /**
     * Load sidebar exclusion if needed
     */
    public function search_exclude_sidebar() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();

        if ( isset( $advanced_settings['search_exclude_sidebar']) && $advanced_settings['search_exclude_sidebar'] === '1' ) {
            add_action( 'wp', array( $this, 'remove_transposh_actions_for_sidebar' ), 999 );
        }        
    }

    /**
     * On the search page, avoid search in sidebars list of posts.
     */
    public function remove_transposh_actions_for_sidebar() {
        // Check if it's a search page and any sidebar has widgets
        if ( is_search() && !is_admin() && $this->has_any_sidebar_widgets_on_template() ) {
            global $my_transposh_plugin;
            $transposh_plugin = $my_transposh_plugin;

            // Remove the actions added by Transposh for the sidebar query
            remove_action( 'pre_get_posts', array( $transposh_plugin, 'pre_post_search' ) );
            remove_action( 'posts_where_request', array( $transposh_plugin, 'posts_where_request' ) );
        }
    }

    /**
     * Check if any sidebar has widgets on the current template.
     *
     * @return bool True if any sidebar has widgets on the current template, false otherwise.
     */
    private function has_any_sidebar_widgets_on_template() {
        global $wp_registered_sidebars, $wp_registered_widgets;

        // Get the widgets assigned to each sidebar
        $sidebar_widgets = wp_get_sidebars_widgets();

        foreach ( $wp_registered_sidebars as $sidebar_id => $sidebar ) {
            // Check if the sidebar has widgets
            if ( isset( $sidebar_widgets[ $sidebar_id ] ) && ! empty( $sidebar_widgets[ $sidebar_id ] ) ) {
                // Check if the sidebar is active on the current template
                if ( $this->is_sidebar_active_on_template( $sidebar_id ) ) {
                    return true; // At least one sidebar has widgets on the current template
                }
            }
        }

        return false; // No sidebar has widgets on the current template
    }

    /**
     * Check if a sidebar is active on the current template.
     *
     * @param string $sidebar_id The ID of the sidebar to check.
     * @return bool True if the sidebar is active on the current template, false otherwise.
     */
    public function is_sidebar_active_on_template( $sidebar_id ) {
        global $wp_registered_sidebars, $wp_registered_widgets;

        // Get the widgets assigned to each sidebar
        $sidebar_widgets = wp_get_sidebars_widgets();

        // Check if the sidebar is registered
        if ( !isset( $wp_registered_sidebars[$sidebar_id] ) ) {
            return false;
        }

        // Check if the sidebar has widgets
        if ( isset( $sidebar_widgets[$sidebar_id] ) && !empty( $sidebar_widgets[$sidebar_id] ) ) {
            // Check if any widget in the sidebar is active in the current template
            foreach ( $sidebar_widgets[$sidebar_id] as $widget_id ) {
                $widget_callback = $wp_registered_widgets[$widget_id]['callback'];

                // Check if the widget is an instance of WP_Widget
                if ( is_array( $widget_callback ) && isset( $widget_callback[0] ) && is_a( $widget_callback[0], 'WP_Widget' ) ) {
                    // Maybe need to add more conditions based on themes' structure
                    if ( is_active_widget( false, false, $widget_callback[0]->id_base, true ) ) {
                        return true;
                    }
                }
            }
        }
        return false;
    }

    /**
     * Hide admin top bar on front-end if necessary.
     */
    public function hide_admin_top_bar() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();

        if ( isset( $advanced_settings['hide_admin_bar']) && $advanced_settings['hide_admin_bar'] === '1' ) {
            add_filter( 'show_admin_bar', '__return_false' );
        }
    }

    /**
     * Manage Translator role.
     */
    public function manage_translator_role() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();

        $role_name = 'tlt_translator';

        $custom_role = get_role( $role_name );

        if ( isset( $advanced_settings['translator_role']) && $advanced_settings['translator_role'] === '1' ) {
            if ( ! $custom_role ) {
                add_role( $role_name, 'TLT Translator', array(
                    'read' => true,
                ) );
            }
        } else {
            if ( $custom_role ) {
                remove_role( $role_name );
                // Remove capability from all users with the "tlt_translator" role
                $users = get_users( array( 'role' => $role_name ) );
                foreach ( $users as $user ) {
                    $user->remove_cap( $role_name );
                }
            }
        }
    }

    /**
     * Restrict admin access for the Translator role.
     */
    public function restrict_admin_access() {
        if ( is_admin() && ! defined( 'DOING_AJAX' ) && current_user_can( 'tlt_translator' ) && ! current_user_can( 'administrator' ) ) {
            wp_redirect( home_url() );
            exit;
        }
    }

    /**
     * Translator role front-end actions.
     */
    public function front_translator_role() {
        $translator_role = get_role( 'tlt_translator' );
        if ( $translator_role && current_user_can( 'tlt_translator' ) && ! current_user_can( 'administrator' ) ) {
            add_filter( 'login_redirect', array( $this, 'custom_login_redirect' ), 10, 3 );
            add_filter( 'show_admin_bar', array( $this, 'hide_admin_bar_for_translator' ) );
            add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_dashicons' ) );
            add_action( 'wp_footer', array( $this, 'add_translator_links' ) );
        }
    }

    /**
     * Enqueue Dashicons stylesheet.
     */
    public function enqueue_dashicons() {
        wp_enqueue_style( 'dashicons' );
    }

    /**
     * Custom login redirect for translators.
     *
     * @param string $redirect_to The redirect URL.
     * @param string $request     The original request URL.
     * @param WP_User $user        The user object.
     * @return string The modified redirect URL.
     */
    public function custom_login_redirect( $redirect_to, $request, $user ) {
        $redirect_to = home_url();
        return $redirect_to;
    }
 
    /**
     * Hide admin bar for translators.
     *
     * @param bool $show Whether to show the admin bar.
     * @return bool Whether to show or hide the admin bar.
     */
    function hide_admin_bar_for_translator( $show ) {
        $show = false;
        return $show;
    }

    /**
     * Add logout and edit translation links to the footer.
     */
    public function add_translator_links() {
        $logout_url = wp_logout_url();
        ?>
        <style>
            .tlt-translator-links {
                position: fixed;
                top: 50%;
                right: 20px;
                transform: translateY(-50%);
                z-index: 9999;
            }
            .tlt-translator-links .tlt-icon-wrapper {
                display: block;
                width: 50px;
                height: 50px;
                margin-bottom: 10px;
                border-radius: 50%;
                background-color: #f9f9f9;
                transition: background-color 0.3s ease;
                cursor: pointer;
                text-decoration: none;
            }
            .tlt-translator-links .tlt-icon {
                display: flex;
                justify-content: center;
                align-items: center;
                width: 100%;
                height: 100%;
                font-size: 24px;
                text-decoration: none;
            }
            .tlt-translator-links .tlt-icon.tlt-edit:before {
                content: '\f464'; /* Dashicons edit icon */
            }
            .tlt-translator-links .tlt-icon.tlt-exit-edit:before {
                content: '\f158'; /* Dashicons no icon */
            }
            .tlt-translator-links .tlt-icon.tlt-logout:before {
                content: '\f14a'; /* Dashicons exit icon */
            }
            .tlt-translator-links .tlt-icon-wrapper:hover {
                background-color: #d9d9d9;
            }
        </style>
        <div class="tlt-translator-links">
            <a href="#" class="tlt-icon-wrapper" onclick="tltEditTranslation()">
                <?php if ( strpos( $_SERVER['REQUEST_URI'], 'tpedit=1' ) !== false ) : ?>
                    <span class="dashicons tlt-icon tlt-exit-edit"></span>
                <?php else : ?>
                    <span class="dashicons tlt-icon tlt-edit"></span>
                <?php endif; ?>
            </a>
            <a href="<?php echo esc_url( $logout_url ); ?>" class="tlt-icon-wrapper">
                <span class="dashicons tlt-icon tlt-logout"></span>
            </a>
        </div>
        <script>
            function tltEditTranslation() {
                var currentUrl = document.location.href;
                var editModeParam = 'tpedit=1';
                var editUrl;
				if (currentUrl.endsWith('#')) {
					currentUrl = currentUrl.slice(0, -1);
				}
                if (currentUrl.includes(editModeParam)) {
                    // Edit mode already enabled, remove the parameter
                    editUrl = currentUrl.replace('?' + editModeParam, '').replace('&' + editModeParam, '');
                } else {
                    // Edit mode not enabled, add the parameter
                    editUrl = currentUrl + (currentUrl.includes('?') ? '&' : '?') + editModeParam;
                }
                window.location.assign(editUrl);
            }
        </script>
        <?php
    }

    /**
     * Get the roles that are allowed to access the admin translation editor.
     *
     * @return array An array of roles with admin access.
     */
    public function get_admin_edit_roles() {
        // Get the roles from options
        $admin_edit_roles = isset( $this->tlt_options['advanced_settings']['admin_edit_roles'] ) ? $this->tlt_options['advanced_settings']['admin_edit_roles'] : array();
        
        return $admin_edit_roles;
    }


    /**
	 * SEO integration, for now, only for Rank Math.
	 */	
    public function seo_integration() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();

        //Rank math integration
        if ( isset( $advanced_settings['rank_math_sitemap_integration'] ) && $advanced_settings['rank_math_sitemap_integration'] === '1' ) {
            if ( class_exists( 'RankMath' ) ) {
                
                // Manage sitemap URL modifications if "Post Language Filtering" option is active
                if ( isset( $advanced_settings['activate_posts_exclude'] ) && $advanced_settings['activate_posts_exclude'] === '1' ) {
                    add_filter( 'rank_math/sitemap/xml_post_url', array( $this, 'modify_sitemap_url' ), 10, 2 );
                }

                // Manage sitemap URL additions
                $selected_post_types = isset( $advanced_settings['sitemap_selected_post_types'] ) ? $advanced_settings['sitemap_selected_post_types'] : array();
                foreach ( $selected_post_types as $post_type ) {
                    add_action( "rank_math/sitemap/{$post_type}_content", function () use ( $post_type ) {
                        $sitemap_content = $this->custom_sitemap_content( $post_type );
                        return( $sitemap_content );
                    } );
                }
            }
        }
    }

    /**
     * Modify sitemap content for a given post.
     *
     * @param string $url  The URL to be modified.
     * @param object $post The post object being processed.
     * @return string Modified URL for the given post type.
     */
    public function modify_sitemap_url( $url, $post ) {
        $filtering_post_types = isset( $this->tlt_options['advanced_settings']['selected_post_types'] ) ? $this->tlt_options['advanced_settings']['selected_post_types'] : array();

        // Check if the current post type is one of the filtered post types
        if ( in_array( $post->post_type, $filtering_post_types ) ) {
            // Get the Transposh language meta value for the post
            $tp_language = get_post_meta( $post->ID, 'tp_language', true );

            // Check if we have to add language code to this URL
            if ( $tp_language !== '' && $tp_language !== false && $tp_language !== $this->default_language ) {
                $language_url = home_url( '/' . $tp_language );
                $modified_url = str_replace( home_url(), $language_url, $url ); 
                return $modified_url;
            }
        }
        return $url;
    }

    /**
     * Generate custom sitemap content for a given post type.
     * @param string $sitemap_type The post type for which to generate the sitemap content.
     * @return array Custom sitemap content for the given post type.
     */
    public function custom_sitemap_content( $type ) {
        // Initialize an empty array to store URL information
        $url_array = array();

        global $wpdb;

        // Define table name
        $posts_table = $wpdb->prefix . 'posts';
        
        // Build the query
        $query = "SELECT * FROM $posts_table WHERE post_type = %s AND post_status = %s";
        $prepared_query = $wpdb->prepare( $query, $type, 'publish' );

        // Execute the query
        $posts = $wpdb->get_results( $prepared_query );

        // Get the selected languages from plugin options
        $languages = isset( $this->tlt_options['advanced_settings']['sitemap_selected_languages'] ) ? $this->tlt_options['advanced_settings']['sitemap_selected_languages'] : array();

        // Get the Post Language Filtering options
        $test_filtering = false;
        $filtering_display_default = false;
        if ( isset( $this->tlt_options['advanced_settings']['activate_posts_exclude'] ) && $this->tlt_options['advanced_settings']['activate_posts_exclude'] === '1' ) {
            $filtering_post_types = isset( $this->tlt_options['advanced_settings']['selected_post_types'] ) ? $this->tlt_options['advanced_settings']['selected_post_types'] : array();

            if ( in_array( $type, $filtering_post_types ) ) {
                $test_filtering = true;
                $filtering_display_default = isset( $this->tlt_options['advanced_settings']['always_display_default'] ) ? ( $this->tlt_options['advanced_settings']['always_display_default'] ? true : false ) : false;
            }
        }

        // Loop through the posts or pages and add the URLs and last modification dates to the array
        foreach ( $posts as $post ) {
            // Check if the post has the "noindex" directive
            $robots_meta = get_post_meta( $post->ID, 'rank_math_robots', true );
            if ( is_array( $robots_meta ) && in_array( 'noindex', $robots_meta ) ) {
                continue; // Skip the post
            }

            // Check if we have to skip the post because of Post Language Filtering options
             if ( $test_filtering ) {
                $tp_language = get_post_meta( $post->ID, 'tp_language', true );
                if ( !$filtering_display_default && ( $tp_language === '' || $tp_language === false ) ) { 
                    continue; // Skip the post, it is set to default language and display_default is false
                }
                if ( $tp_language !== '' && $tp_language !== false && $tp_language !== $this->default_language ) { 
                    continue; // Skip the post, we will add language code to the default url using 'rank_math/sitemap/xml_post_url'
                }
            }

            // Get the last GMT modification date in Unix timestamp format
            $post_modified_gmt = strtotime( $post->post_modified_gmt );
            // Create a DateTime object with the GMT time
            $gmt_date_time = new \DateTime( "@$post_modified_gmt" );
            // Get the website's time zone setting from WordPress
            $wordpress_time_zone = get_option( 'timezone_string' );
            $wordpress_time_zone_object = new \DateTimeZone( $wordpress_time_zone );
            $gmt_date_time->setTimezone( $wordpress_time_zone_object );
            // Format the date in the WordPress time zone
            $last_modified_utc = $gmt_date_time->format( 'c' );

            // Loop through selected languages and modify URLs accordingly
            foreach ( $languages as $language ) {
                $language_url = home_url( '/' . $language );
                $url_array[] = array(
                    'url'     => str_replace( home_url(), $language_url, get_permalink( $post->ID ) ),
                    'lastmod' => $last_modified_utc,
                );
            }
        }

        // Create the sitemap content string
        $sitemap_content = '';
        //foreach ( $en_url_array as $url_info ) {
        foreach ( $url_array as $url_info ) {
            $sitemap_content .= "\t" . '<url>' . PHP_EOL .
                "\t\t" . '<loc>' . esc_url( $url_info['url'] ) . '</loc>' . PHP_EOL .
                "\t\t" . '<lastmod>' . $url_info['lastmod'] . '</lastmod>' . PHP_EOL .
                "\t" . '</url>' . PHP_EOL;
        }
        return $sitemap_content;
    }

    /**
     * Check if "Conditional Display" option is enabled.
     *
     * @return bool True if the option is enabled, false otherwise.
     */
    public function is_conditional_display_enabled() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();
        return isset( $advanced_settings['conditional_display'] ) && $advanced_settings['conditional_display'] === '1';
    }

    /**
     * Generates the conditional display CSS based on Transposh viewable languages.
     *
     * @return string The CSS code.
     */
    public function get_conditional_display_css() {
        $css_code = '.tlt-show-' . str_replace(',', ',.tlt-show-', $this->viewable_languages) . ' {display:none;} ';
        
        $viewable_languages_array = explode( ',', $this->viewable_languages );
        foreach ( $viewable_languages_array as $index => $language_code ) {
            $language_code = trim( $language_code );
            $css_code .= 'html[lang="' . $language_code . '"]' . ' .tlt-show-' . $language_code . ' {display:inline-block;} ';
            $css_code .= 'html[lang="' . $language_code . '"]' . ' .tlt-hide-' . $language_code . ' {display:none;} ';
        }
        $css_code = $this->simple_minify( $css_code );
        return $css_code;
    }

    /**
     * Check if the "Remove Empty HTML Elements" option is enabled in the advanced settings.
     *
     * @return bool True if the option is enabled, false otherwise.
     */
    public function is_remove_empty_html_elements_enabled() {
        $advanced_settings = isset( $this->tlt_options['advanced_settings'] ) ? $this->tlt_options['advanced_settings'] : array();
        return isset( $advanced_settings['remove_empty_html_elements'] ) && $advanced_settings['remove_empty_html_elements'] === '1';
    }

    /**
     * Generates the "Remove Empty HTML Elements" JS.
     *
     * @return string The JS code.
     */
    public function get_remove_empty_html_elements_js() {
        $js_code = '
            var tltParagraphs = document.querySelectorAll("p");
            var tltSpans = document.querySelectorAll("span");
        
            /* Loop through all <p> tags and remove if empty and has no attributes */
            for (var i = 0; i < tltParagraphs.length; i++) {
                var tltParagraph = tltParagraphs[i];
                if (tltParagraph.textContent === "" && !tltParagraph.hasAttributes() && tltParagraph.children.length === 0) {
                    tltParagraph.parentNode.removeChild(tltParagraph);
                }
            }
        
            /* Loop through all <span> tags and remove if empty and has no attributes */
            for (var j = 0; j < tltSpans.length; j++) {
                var tltSpan = tltSpans[j];
                if (tltSpan.textContent === "" && !tltSpan.hasAttributes() && tltSpan.children.length === 0) {
                    tltSpan.parentNode.removeChild(tltSpan);
                }
            }
        ';
        $js_code = $this->simple_minify( $js_code );
        return $js_code;
    }

	/**
	 * Register the settings
	 */		
    public function set_settings() {
        register_setting( 'tlt_advanced_settings', TRANSPOSH_TOOLBOX_OPTION_NAME, array ( $this, 'register_settings_callback' ) );
    }

	/**
	 * Add sections to settings page
	 */
    public function set_sections() {
        add_settings_section( 'tlt_section_excerpt', esc_html__( 'Excerpt Length', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'excerpt_section_callback' ), 'transposh-language-toolbox', array( 'before_section' => '<div id="tlt-post-settings" class="tlt-tab-content active">' ) );
        add_settings_section( 'tlt_section_posts_exclude', esc_html__( 'Post Language Filtering', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'posts_exclude_section_callback' ), 'transposh-language-toolbox' );
        add_settings_section( 'tlt_section_search_sidebar', esc_html__( 'Search Page Posts', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'search_sidebar_section_callback' ), 'transposh-language-toolbox', array( 'after_section' => '</div>' ) );
        add_settings_section( 'tlt_section_hide_admin_bar', esc_html__( 'Hide Admin Bar', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'hide_admin_bar_section_callback' ), 'transposh-language-toolbox', array( 'before_section' => '<div id="tlt-translate-options" class="tlt-tab-content">' ) );
        add_settings_section( 'tlt_section_translator_role', esc_html__( 'Translator Role', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'translator_role_section_callback' ), 'transposh-language-toolbox' );
        add_settings_section( 'tlt_section_admin_edit_roles', esc_html__( 'Admin Roles', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'admin_edit_roles_section_callback' ), 'transposh-language-toolbox', array( 'after_section' => '</div>' ) );
        add_settings_section( 'tlt_section_seo_integration', esc_html__( 'SEO Integration', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'seo_integration_section_callback' ), 'transposh-language-toolbox', array( 'before_section' => '<div id="tlt-seo-integration" class="tlt-tab-content">', 'after_section' => '</div>' ) );
        add_settings_section( 'tlt_section_conditional_display', esc_html__( 'Conditional Display', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'conditional_display_section_callback' ), 'transposh-language-toolbox', array( 'before_section' => '<div id="tlt-other-settings" class="tlt-tab-content">' ) );
        add_settings_section( 'tlt_section_remove_empty_html_elements', esc_html__( 'Remove Empty HTML Elements', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( $this, 'remove_empty_html_elements_section_callback' ), 'transposh-language-toolbox', array( 'after_section' => '</div>' ) );
    }

	/**
	 * Add fields to settings page
	 */	
    public function set_fields() {
        $args = array(
            array(
				'id' => 'field_activate_excerpt_length',
				'title' => esc_html__( 'Activate custom excerpt length', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_excerpt',
				'args' => array(
					'label_for' => 'activate_excerpt_length',
					'description' => esc_html__( 'If activated, the excerpts will be cut according to the settings below.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
				'id' => 'field_excerpt_length',
				'title' => esc_html__( 'Excerpt length for alphabetic languages', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_number_callback' ),
				'section' => 'tlt_section_excerpt',
				'args' => array(
					'label_for' => 'excerpt_length',
                    'positive' => true,
					'description' => esc_html__( 'Specify the number of words.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
                'id' => 'field_excerpt_length_special',
                'title' => esc_html__( 'Excerpt length for ideographic languages', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                'callback' => array( $this, 'field_number_callback' ),
                'section' => 'tlt_section_excerpt',
                'args' => array(
                    'label_for' => 'excerpt_length_special',
                    'positive' => true,
                    'description' => __( 'Specify the number of ideograms.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
				'id' => 'field_activate_posts_exclude',
				'title' => esc_html__( 'Exclude posts based on language', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_posts_exclude',
				'args' => array(
					'label_for' => 'activate_posts_exclude',
					'description' => esc_html__( 'If activated, post lists will output only the posts of the current language.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
                'id' => 'field_selected_post_types',
                'title' => esc_html__( 'Posts/Custom Posts', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                'callback' => array( $this, 'field_multi_select_callback' ),
                'section' => 'tlt_section_posts_exclude',
                'args' => array(
                    'label_for' => 'selected_post_types',
                    'description' => esc_html__( 'Select the posts/custom post types to activate the language filter on.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'select_options' => $this->get_post_types( false ),
                )
            ),
            array(
				'id' => 'field_always_display_default',
				'title' => esc_html__( 'Always display posts without assigned language', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_posts_exclude',
				'args' => array(
					'label_for' => 'always_display_default',
					'description' => esc_html__( 'By default, posts without a language assignment are only displayed on the default language pages. Activating this option will display these posts on all languages.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),  
                )
            ),
            array(
				'id' => 'field_search_exclude_sidebar',
				'title' => esc_html__( 'Exclude all sidebars from Transposh search', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_search_sidebar',
				'args' => array(
					'label_for' => 'search_exclude_sidebar',
					'description' => esc_html__( 'If activated, the search will not impact post lists in any sidebar on the search page.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
				'id' => 'field_hide_admin_bar',
				'title' => esc_html__( 'Hide admin bar', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_hide_admin_bar',
				'args' => array(
					'label_for' => 'hide_admin_bar',
					'description' => esc_html__( 'Hide admin top bar on the front end.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
				'id' => 'field_translator_role',
				'title' => esc_html__( 'Activate translator role', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_translator_role',
				'args' => array(
					'label_for' => 'translator_role',
					'description' => esc_html__( 'Create the "TLT Translator" role.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
                'id' => 'field_admin_edit_roles',
                'title' => esc_html__( 'Translation editors', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                'callback' => array( $this, 'field_multi_select_callback' ),
                'section' => 'tlt_section_admin_edit_roles',
                'args' => array(
                    'label_for' => 'admin_edit_roles',
                    'description' => esc_html__( 'Choose the roles allowed to edit translations in admin area.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'select_options' => $this->get_all_roles(),
                )
            ),
            array(
				'id' => 'field_rank_math_sitemap_integration',
				'title' => esc_html__( 'Activate RankMath sitemap integration', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_seo_integration',
				'args' => array(
					'label_for' => 'rank_math_sitemap_integration',
					'description' => esc_html__( 'Activate this to add the languages URLs to RankMath sitemap.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
                'id' => 'field_sitemap_selected_languages',
                'title' => esc_html__( 'Languages', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                'callback' => array( $this, 'field_multi_select_callback' ),
                'section' => 'tlt_section_seo_integration',
                'args' => array(
                    'label_for' => 'sitemap_selected_languages',
                    'description' => esc_html__( 'Select the languages you wish to include in the sitemap.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'select_options' => $this->get_sitemap_languages(),
                )
            ),
            array(
                'id' => 'field_sitemap_selected_post_types',
                'title' => esc_html__( 'Posts/Custom Posts', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                'callback' => array( $this, 'field_multi_select_callback' ),
                'section' => 'tlt_section_seo_integration',
                'args' => array(
                    'label_for' => 'sitemap_selected_post_types',
                    'description' => esc_html__( 'Select the posts/custom post types you wish to include in the sitemap.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'select_options' => $this->get_post_types( true ),
                )
            ),
            array(
				'id' => 'field_conditional_display',
				'title' => esc_html__( 'Activate Conditional Display', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_conditional_display',
				'args' => array(
					'label_for' => 'conditional_display',
					'description' => esc_html__( 'Generate conditional display classes.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
            array(
				'id' => 'field_remove_empty_html_elements',
				'title' => esc_html__( 'Remove Empty Tags', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
				'callback' => array( $this, 'field_checkbox_callback' ),
				'section' => 'tlt_section_remove_empty_html_elements',
				'args' => array(
					'label_for' => 'remove_empty_html_elements',
					'description' => esc_html__( 'Enable this option to remove empty HTML elements from the DOM.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN )
                )
            ),
        );

        foreach ( $args as $arg ) {
            add_settings_field( $arg['id'], $arg['title'], $arg['callback'], 'transposh-language-toolbox', $arg['section'], $arg['args'] );
        }
    }

    /**
     * Get available post types.
     *
     * @param bool $include_pages Whether to include pages or not.
     * @return array An array of post types.
     */
    private function get_post_types( $include_pages = true ) {
        $post_types = get_post_types( array( 'public' => true ), 'objects' );
        $allowed_post_types = array();

        foreach ( $post_types as $post_type ) {
            if ( $post_type->name !== 'attachment' && ( $include_pages || $post_type->name !== 'page' ) ) {
                $allowed_post_types[ $post_type->name ] = $post_type->label; // Transform object into associative array
            }
        }

        return $allowed_post_types;
    }

    /**
     * Get a list of all WordPress roles.
     *
     * @return array List of role names and labels.
     */
    public function get_all_roles() {
        $all_roles = wp_roles()->roles;
        $exclude_roles = array('administrator', 'subscriber', 'tlt_translator');
        $role_names = array();

        foreach ( $all_roles as $role_name => $role_info ) {
            // Check if the role is not in the exclude list
            if ( !in_array( $role_name, $exclude_roles ) ) {
                $role_names[$role_name] = $role_info['name'];
            }
        }

        return $role_names;
    }

    /**
     * Get languages selection for sitemap.
     *
     * @return array An array of languages.
     */
    private function get_sitemap_languages() {
        $sitemap_languages = explode( ',', $this->viewable_languages );
 
        // Remove the default language from the array
        $default_language_index = array_search( $this->default_language, $sitemap_languages );
        if ( $default_language_index !== false ) {
            unset( $sitemap_languages[ $default_language_index ] );
        }
    
        // Transform the array into an associative array
        $sitemap_languages = array_combine( $sitemap_languages, $sitemap_languages );

        return $sitemap_languages;
    }

    /**
     * Callback function for registering and saving the settings
     *
     * @param array $input The submitted settings data
     * @return array The sanitized and merged settings data
     */
    public function register_settings_callback( $input ) {
        if ( isset( $_POST['tlt_custom_languages_form_submitted'] ) && $_POST['tlt_custom_languages_form_submitted'] === '1' ) {
            return Custom_Languages::$cached_tlt_options;
        }
        // Retrieve the existing options
        $existing_options = get_option( TRANSPOSH_TOOLBOX_OPTION_NAME, array() );

        $output = array();

        // Process and sanitize the submitted settings data
        foreach ( $input['advanced_settings'] as $key => $value ) {
            if ( ( $key === 'excerpt_length' || $key === 'excerpt_length_special' ) ) {
                // Check for empty input
                if ( $value === null || $value === '' ) {
                    if ( isset( $input['advanced_settings']['activate_excerpt_length'] ) && $input['advanced_settings']['activate_excerpt_length'] == 1 ) {
                        add_settings_error(
                            TRANSPOSH_TOOLBOX_OPTION_NAME,
                            'invalid_empty',
                            esc_html__( 'Empty input: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . $key,
                            'warning'
                        );
                    } 
                    continue;
                }
                
                // Check for non-positive integer input
                if ( ! $this->is_positive_integer( $value ) ) {
                    add_settings_error(
                        TRANSPOSH_TOOLBOX_OPTION_NAME,
                        'invalid_input',
                        esc_html__( 'Invalid input: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . $value,
                        'error'
                    );
                    continue;
                }

                // Check if activate_excerpt_length is not activated
                if ( ! isset( $input['advanced_settings']['activate_excerpt_length'] ) || $input['advanced_settings']['activate_excerpt_length'] != 1 ) {
                    add_settings_error(
                        TRANSPOSH_TOOLBOX_OPTION_NAME,
                        'invalid_excerpt_activation',
                        esc_html__( 'Excerpt length activation is required: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . $key,
                        'error'
                    );
                    continue;
                }
            }
            $output[$key] = $value;
        }

        // Return the sanitized settings data
        $existing_options['advanced_settings'] = $output;

        return $existing_options;
    }

    /**
	 * Output sections description or additional content
	 */	
    public function excerpt_section_callback() {
        echo( '<p>' . esc_html__( 'To avoid conflicts with certain themes or plugins that modify excerpt length, activate this option. This gives you control over excerpt length for both alphabetic languages (those with word spaces) and ideographic languages (those without spaces).', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'The ideographic option covers: "Han" languages (Chinese, Japanese, Korean), "Thai", "Khmer", "Lao", "Myanmar", "Tibetan".', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'When providing a single excerpt length value, the other will be automatically calculated: ideographic languages\' excerpt length will be double that of alphabetic languages.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }
    public function posts_exclude_section_callback() {
        echo( '<p>' . esc_html__( 'Enable this option to display posts and custom post types according to current language. Only posts set to the current language will show up in loops, including archive pages and sections with lists of posts or custom post types.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'To assign a language to a post, edit the post and find the Transposh "Set post language" option. Select the desired language for the post.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'By default, posts without a specific language assignment are visible only on default language pages. Enabling "Always display posts without assigned language" will show these posts on all language pages. This lets your website feature both single-language and multi-language posts.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'Keep in mind that this function follows WordPress best practices, which are compatible with most themes and plugins. However, certain themes and plugins may use alternative methods that could be incompatible with this feature.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'Additionally, note that on a post\'s page assigned to a specific language, language menu items for other languages will be disabled for custom shortcodes from this plugin (those starting with "tlt"). For Transposh shortcodes, the language menu will be entirely hidden.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }
    public function search_sidebar_section_callback() {
        echo( '<p>' . esc_html__( 'When the Transposh option "Enable search in translated languages" is active, a list of posts in the search page sidebar can be filtered by the search query. To prevent this behavior, enable the following option.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }
    public function hide_admin_bar_section_callback() {
        echo( '<p>' . esc_html__( 'Activate this option if you use Transposh automatic translation and want to exclude the admin top bar from being translated. By hiding the admin bar, you\'ll avoid unnecessary translation of its content, reducing stored strings in the database.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'Note: After automatic translation, you can use the Transposh utility "Delete automated translations that add no apparent value" to remove lines with identical original and translated strings. Find this in the Transposh Utilities tab. Alternatively, you can use the Translator Editor of this plugin and click the "Identical Translations Finder" button to review these rows before deletion.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }
    public function translator_role_section_callback() {
        echo( '<p>' . esc_html__( 'Activate the "TLT Translator" option to create a front-end translator role without admin access. You will need to assign translation capabilities to this role in the Transposh "Who can translate" option, and apply the role to specific user(s).', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'Translators will see two icons on the front-end: "Edit Translation" and "Logout". Clicking "Edit Translation" enables Transposh translation mode. Alternatively, translators can also access translation mode by adding "?tpedit=1" to the page URL.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
        echo( '<p>' . esc_html__( 'Note: Deactivating this option removes the "TLT Translator" role from WordPress, and from any user(s) who previously had it.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }
    public function admin_edit_roles_section_callback() {
        echo( '<p>' . esc_html__( 'Choose the WordPress roles that can access the Language Toolbox Translation Editor. Administrators have access by default, while TLT Translators do not, as they cannot access the admin area.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }
    public function seo_integration_section_callback() {
        echo '<p>' . esc_html__( 'This feature works only with RankMath. By default Rank Math doesn\'t include language-specific URLs from Transposh in its sitemap. Activate this option to add your Transposh-created URLs to the sitemap. Posts marked as "noindex" won\'t be included. If a post type isn\'t included in Rank Math\'s sitemap configuration, the corresponding language URLs won\'t appear in the sitemap.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>';
        echo( '<p>' . esc_html__( 'Note: Enabling "Post Language Filtering" (in Transposh Language Toolbox\'s "Post Settings") makes the sitemap display only URLs in the post\'s configured language.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }
    public function conditional_display_section_callback() {
        echo '<p>' . wp_kses( __( 'This function generates inline CSS classes that allow you to show or hide elements depending on the active language. The classes are: <code>tlt-show-{code}</code> and <code>tlt-hide-{code}</code>, where {code} is a language code. Example for the languages "en", "es", and "fr": if the current language is English, elements with the class <code>tlt-show-en</code> will be displayed, elements with the classes <code>tlt-show-es tlt-show-fr</code> will be hidden, elements with the class <code>tlt-hide-en</code> will be hidden, and elements with the classes <code>tlt-hide-es tlt-hide-fr</code> will be displayed.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( 'code' => array() ) ) . '</p>';
        $css_classes = 'tlt-show-' . str_replace(',', ', tlt-show-', $this->viewable_languages);
        $css_classes .= ', tlt-hide-' . str_replace(',', ', tlt-hide-', $this->viewable_languages) . '.';
        echo( '<p>' . esc_html__( 'Based on your current Transposh languages, the classes generated will be: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) );
        echo('<code>' . $css_classes . '</code>' . '</p>');
    }
    public function remove_empty_html_elements_section_callback() {
        echo '<p>' . wp_kses( __( 'When using Transposh conditional display like <code>&lt;span lang="en" class="only_thislanguage"&gt;Some text&lt;/span&gt;</code> or <code>[tp not_in="en"]Some text[/tp]</code> to hide content based on language, the text will be hidden but might remain empty html element such as &lt;p&gt;&lt;/p&gt;, which could cause design issues. Activate this feature to automatically remove those empty tags.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array( 'code' => array() ) ) . '</p>';
        echo( '<p>' . esc_html__( 'Please note: Only empty &lt;p&gt; and &lt;span&gt; will be removed.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</p>' );
    }

    /**
	 * Output the field HTML
	 */	
    public function field_number_callback( $args ) {
        $option_name = $args['label_for'];
        $positive = $args['positive'];
        $description = $args['description'];

        $number = isset( $this->tlt_options['advanced_settings'][$option_name] ) ? $this->tlt_options['advanced_settings'][$option_name] : null; 
        if ( $positive === true ) {
            if ( ! $this->is_positive_integer( $number ) ) {
			    $number = null;
		    }
        }
        ?> 
        <input id="<?php echo( $option_name ); ?>" type="number" name="<?php echo(TRANSPOSH_TOOLBOX_OPTION_NAME); ?>[advanced_settings][<?php echo( $option_name ); ?>]" value="<?php echo( $number ); ?>" <?php if ( $positive ) { echo 'min="1"';} ?> > <span class="tlt-settings-description"><?php echo( $description ); ?></span> 
        <?php
    }
	public function field_checkbox_callback( $args ) {
        $option_name = $args['label_for'];
        $description = $args['description'];
		$checked = isset( $this->tlt_options['advanced_settings'][$option_name] ) ? ( $this->tlt_options['advanced_settings'][$option_name] ? true : false ) : false;
        ?>
		<input type="checkbox" id="<?php echo( $option_name ); ?>" name="<?php echo(TRANSPOSH_TOOLBOX_OPTION_NAME); ?>[advanced_settings][<?php echo( $option_name ); ?>]" value="1" class="" <?php echo( $checked ? 'checked' : '' ); ?> ><label for="<?php echo( $option_name ); ?>" class="tlt-settings-description" ><?php echo( $description ); ?></label>
        <?php
    }
    public function field_multi_select_callback( $args ) {
        $option_name = $args['label_for'];
        $description = $args['description'];
        $select_options = $args['select_options'];
    
        $selected_options = isset( $this->tlt_options['advanced_settings'][$option_name] ) ? $this->tlt_options['advanced_settings'][$option_name] : array();

        ?> 
        <select id="<?php echo $option_name; ?>" class="tlt-multi-select" name="<?php echo(TRANSPOSH_TOOLBOX_OPTION_NAME); ?>[advanced_settings][<?php echo $option_name; ?>][]" multiple="multiple">
            <?php foreach ( $select_options as $value => $label ) { ?>
                <option value="<?php echo $value; ?>" <?php selected( in_array( $value, $selected_options ), true ); ?>><?php echo $label; ?></option>
            <?php } ?>
        </select>
        <span class="tlt-settings-description"><?php echo $description; ?></span>
        <?php
    }

    /**
	 * Test if positive integer
	 * @param  string $value The variable to test
	 * @return boolean  The result true or false  
	 */	
	public function is_positive_integer( $value ) {
		if ( ( is_int( $value ) || ctype_digit( $value ) ) && (int)$value > 0 ) {
			return true;
		}
		return false;
	}

    /**
	 * Display the page
	 */	
    public function render_form() {
        settings_errors();
        ?>
        <div class="tlt-wrap">
            <div class="tlt-menu">
                <a href="#" class="tlt-menu-item active" data-tab="tlt-post-settings"><?php esc_html_e( 'Post Settings', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-translate-options"><?php esc_html_e( 'Translate Options', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-seo-integration"><?php esc_html_e( 'SEO Integration', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-other-settings"><?php esc_html_e( 'Other Settings', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
            </div>
            <form id="tlt-advanced-settings-form" method="post" action="options.php">
                <?php 
                settings_fields('tlt_advanced_settings');
                do_settings_sections('transposh-language-toolbox');
                submit_button(); 
                ?>
            </form>
        </div>
        <?php
    }
}