<?php
/**
 * Custom Languages Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

use Transposh_Language_Toolbox\Base_Utilities;

/**
 * Class Add_Language
 *
 * A class for adding a new language to the Transposh plugin options.
 */
class Custom_Languages extends Base_Utilities {

    private static $custom_languages = array();
    public static $cached_tlt_options = array();

    /**
     * A static method for modifying the languages list in the Transposh plugin.
     * This is used in the plugin main file to modify Transposh constants as soon as possible (before classes instantiation).
     */
    public static function modify_transposh_languages() {
        // Load the Transposh constants file
        require_once WP_PLUGIN_DIR . '/transposh-translation-filter-for-wordpress/core/constants.php';

        // Retrieve the plugin options. (We can't use Base_Utilities options here because it's a static method and Base_Utilities in not accessible yet.)
        $options = get_option( TRANSPOSH_TOOLBOX_OPTION_NAME );

         // Set the custom languages from the options
        self::$custom_languages = isset( $options['custom_languages'] ) ? $options['custom_languages'] : array();

        // Modify the languages list
        if ( class_exists( 'transposh_consts' ) ) {
            // Add your custom language
            foreach ( self::$custom_languages as $key => $value ) {
                \transposh_consts::$languages[$key] = $value;
            }
        }
    }

    /**
     * Validate a language code.
     *
     * Language code format: lowercase letters, 2 or 3 characters, optionally followed by subtags or extended language ranges,
     * separated by hyphens.
     * 
     * @param string $languageCode The language code to validate.
     * @return bool Whether the language code is valid or not.
     */
    private function validate_language_code( $language_code ) {
        $pattern = '/^[a-z]{2,3}(-[a-zA-Z0-9]+)*$/';
        // Check if the language code matches the pattern
        if ( preg_match( $pattern, $language_code ) ) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Add a settings error notice for the Custom Languages page.
     *
     * @param string $error_code     The error code or identifier for the notice.
     * @param string $notice_message The error message to be displayed.
     * @param string $notice_level   The notice type (e.g., error, updated, notice, warning).
     */
    public function print_notice( $error_code, $notice_message, $notice_level ) {
        add_settings_error(
            'custom_languages_notice', // Unique identifier for the notice
            $error_code, // Error code or identifier
            $notice_message, // Error message
            $notice_level // Notice type (e.g., error, updated, notice, warning)
        );
    }

    /**
     * Render the Custom Languages page
     */  
    public function render_custom_languages() {

        // Retrieve the plugin options
        $options = $this->tlt_options;
        self::$cached_tlt_options = $options;
        $before_languages = isset( $options['custom_languages'] ) ? $options['custom_languages'] : array();
        $after_languages = array();
        $notice_message = "";
        $notice_level = "";
        $error_code = "";

        // Check if the form was submitted
        if ( isset($_POST['tlt_custom_languages_form_submitted'] ) && $_POST['tlt_custom_languages_form_submitted'] === '1' ) {
            // Verify the nonce
            if ( isset( $_POST[ 'custom_languages_nonce' ] ) && wp_verify_nonce( $_POST[ 'custom_languages_nonce' ], 'custom_languages_action' ) ) {
                if (isset($_POST['tlt-custom_languages_keys']) && isset($_POST['tlt-custom_languages_values'])) {
                    // Form submitted with non-empty values
                    $keys = $_POST['tlt-custom_languages_keys'];
                    $values = $_POST['tlt-custom_languages_values'];
                    $sanitizedPairs = array();
                    $emptyPairs = false;

                    // Sanitize
                    foreach ( $keys as $index => $key ) {
                        // Sanitize each key-value pair using appropriate sanitization functions
                        $sanitizedKey = sanitize_text_field( $key );
                        $sanitizedValue = sanitize_text_field( $values[$index] );

                        // Remove the spaces
                        $sanitizedKey = trim( $sanitizedKey );
                        $sanitizedValue = trim( $sanitizedValue );

                        // Remove spaces before and after each comma
                        $sanitizedValue = str_replace( ', ', ',', $sanitizedValue );
                        $sanitizedValue = str_replace( ' ,', ',', $sanitizedValue );

                        // Add the sanitized pair to the array
                        $sanitizedPairs[] = array(
                            'key' => $sanitizedKey,
                            'value' => $sanitizedValue
                        );

                        // Sanitize more
                        if ( !empty( $sanitizedKey ) && !empty( $sanitizedValue ) ) {
                            // Value contains at least one comma
                            if ( strpos( $sanitizedValue, ',' ) !== false ) {
                                // Validate the key
                                if ( $this->validate_language_code( $sanitizedKey) ) {
                                    $after_languages[$sanitizedKey] = $sanitizedValue;
                                } else {
                                    $error_code = 'custom_languages_key_invalid';
                                    $notice_message = esc_html__( 'Invalid language code: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . $sanitizedKey . esc_html__( '. Language code format should consist of two or three lowercase letters, followed by optional subtags or extended language ranges, separated by hyphens (e.g., en, en-US, zh-TW).', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );
                                    $notice_level = 'notice-warning';
                                    $this->print_notice( $error_code, $notice_message, $notice_level );
                                }                    
                            } else {
                                // Value does not contain any comma
                                if ( isset( $before_languages[$sanitizedKey] ) ) {
                                    $after_languages[$sanitizedKey] = $before_languages[$sanitizedKey];
                                }
                                $error_code = 'custom_languages_value_no_coma';
                                $notice_message = esc_html__( 'Value without coma for: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . $sanitizedKey;
                                $notice_level = 'notice-warning';
                                $this->print_notice( $error_code, $notice_message, $notice_level );
                            }
                        } else {
                            $emptyPairs = true;
                        }
                    }

                    // Add a settings error notice for empty key-value pairs
                    if ( $emptyPairs ) {
                        $error_code = 'custom_languages_empty_pairs';
                        $notice_message = esc_html__( 'Empty key or value detected.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );
                        $notice_level = 'notice-warning';
                        $this->print_notice( $error_code, $notice_message, $notice_level );
                    }
                }
                // Put back languages if they were deleted but still are in transposh viewable languages
                $deleted_languages = array_diff_key( $before_languages, $after_languages );
                $transposh_options = $this->tp_options;
                foreach ( $deleted_languages as $language => $value ) {
                    if ( isset( $transposh_options['viewable_languages'] ) ) {
                        $language_codes = explode( ',', $transposh_options['viewable_languages'] );
                        if ( ( $key = array_search( $language, $language_codes ) ) !== false) {
                            $after_languages[$language] = $before_languages[$language];
                            $error_code = 'viewable_language_error';
                            $notice_message = esc_html__('Deleted language is a Transposh selected language. Please first unselect the language in Transpoh options.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN);
                            $notice_level = 'notice-warning';
                            $this->print_notice( $error_code, $notice_message, $notice_level );
                        }
                    }
                }
                // Update the options based on the custom languages
                if ( empty( $after_languages ) ) {
                    unset( $options['custom_languages'] );
                    self::$custom_languages = array(); // Reset the custom languages array

                    $error_code = 'all_languages_removed';
                    $notice_message = esc_html__( 'All languages have been removed.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );
                    $notice_level = 'notice-info';
                    $this->print_notice( $error_code, $notice_message, $notice_level );
                } else {
                    // Check if the new languages exist in Transpoh sorted_languages
                    $language_codes = explode( ',', $transposh_options['sorted_languages'] );
                    $existing_languages = [];

                    foreach ( $after_languages as $language_key => $language_value ) {
                        if ( in_array( $language_key, $language_codes ) ) {
                            if ( !array_key_exists( $language_key, $before_languages ) ) {
                                unset( $after_languages[$language_key] ); // Remove from $after_languages only if it is not our custom language
                                $existing_languages[] = $language_key;
                            }
                        }
                    }

                    if ( !empty( $existing_languages ) ) {
                        $error_code = 'language_already_exists';
                        $notice_message = esc_html__( 'The following language(s) already exist(s): ' ) . implode( ', ', $existing_languages );
                        $notice_level = 'notice-warning';
                        $this->print_notice( $error_code, $notice_message, $notice_level );
                    }
                    
                    $options['custom_languages'] = $after_languages;
                    self::$custom_languages = $after_languages;

                    // Update the sorted_languages option
                    $language_codes = array_merge( $language_codes, array_keys( $after_languages ) );
                    // Remove duplicates and empty values from the array
                    $language_codes = array_filter( array_unique( $language_codes ) );
                    // Update the sorted_languages option
                    $transposh_options['sorted_languages'] = implode( ',', $language_codes );
                    
                    if ( !empty( $after_languages ) ) {
                        $error_code = 'custom_languages_updated';
                        $notice_message = esc_html__( 'Languages updated!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );
                        $notice_level = 'notice-success';
                        $this->print_notice( $error_code, $notice_message, $notice_level );
                    } else {
                        unset( $options['custom_languages'] );
                    }
                }
                self::$cached_tlt_options = $options; 
                update_option( TRANSPOSH_TOOLBOX_OPTION_NAME, $options );

                // If necessary deleted a custom language that was deleted but still in Transposh sorted_languages 
                $deleted_languages = array_diff_key( $before_languages, $after_languages ); // Refresh in case things were updated in the foreach above
                $language_codes = explode( ',', $transposh_options['sorted_languages'] );

                foreach ( $deleted_languages as $language => $value ) {
                    if ( ( $key = array_search( $language, $language_codes ) ) !== false ) {
                        unset( $language_codes[$key] );
                    }
                }
                
                $transposh_options['sorted_languages'] = implode( ',', $language_codes );
                update_option( TRANSPOSH_TOOLBOX_TP_OPTION_NAME, $transposh_options );

            } else {
                // Nonce is invalid, set an error notice
                $error_code = 'custom_languages_nonce';
                $notice_message = esc_html__( 'Nonce verification failed. Please try again.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );
                $notice_level   = 'error';
                $this->print_notice( $error_code, $notice_message, $notice_level );
            }
        }

        // Render the page content
        ?>
        <div class="tlt-wrap">
            <div class="tlt-menu">
                <a href="#" class="tlt-menu-item active" data-tab="tlt-custom-languages"><?php esc_html_e( 'Custom Languages', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-transposh-flags"><?php esc_html_e( 'Transposh Flags', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-custom-languages-help"><?php esc_html_e( 'Help', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
            </div>

            <!-- Custom languages section -->
            <div id="tlt-custom-languages" class="tlt-tab-content active">
                <h2><?php esc_html_e( 'Custom Languages', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                <div class="tlt-section">
                    <p><?php esc_html_e( 'On this page, you can create languages not present in Transposh. Click "Add New Language" to add a new language entry. Use "Modify" to edit the language details. After editing, click "Update". Remember to click "Save Changes" when you\'re done. You can add multiple languages.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>  
                    <p><?php echo wp_kses( __( 'The <strong>Key</strong> represents the <code>language code</code>. The <strong>Value</strong> should follow Transposh format: <code>English Name, Native Name, Flag Code, Local Code</code> (separated by commas). For instance, use the key: <code>pt-br</code>, with the value: <code>Brazilian Portuguese,Português do Brasil,br,pt_BR</code>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array(), 'code' => array() ) ); ?></p>
                    <p> <?php esc_html_e( 'For more information, refer to the "Help" tab. View all Transpoh flags and codes in the "Transposh Flags" tab.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div><br>

                <?php settings_errors('custom_languages_notice'); ?>
                <form id="tlt-custom-languages-form" method="post" action="">
                    <div id="tlt-custom-languages-container">
                        <?php foreach (self::$custom_languages as $key => $value) { ?>
                            <div class="tlt-custom-language-row">
                                <div class="tlt-row-item"> 
                                    <label><?php esc_html_e( 'Key:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></label>
                                    <input type="text" class="tlt-field-key" name="tlt-custom_languages_keys[]" value="<?php echo $key; ?>" readonly>
                                </div>
                                <div class="tlt-row-item">
                                    <label><?php esc_html_e( 'Value:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></label>
                                    <input type="text" class="tlt-field-value" name="tlt-custom_languages_values[]" value="<?php echo $value; ?>" readonly>
                                </div>
                                <div class="tlt-row-item">                        
                                    <button type="button" class="tlt-edit-language button action"><?php esc_html_e( 'Modify', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) ?></button>
                                    <button type="button" class="tlt-delete-language button action"><?php esc_html_e( 'Delete', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) ?></button>
                                </div>
                            </div>
                        <?php } ?>
                    </div>
                    <br>
                    <input type="hidden" name="tlt_custom_languages_form_submitted" value="1">
                    <?php wp_nonce_field( 'custom_languages_action', 'custom_languages_nonce' ); ?>
                    <button type="button" id="tlt-add-language" class="button action"><?php esc_html_e( 'Add New Language', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) ?></button>
                    <input type="submit" id="tlt-save-button" class="button-primary" value="Save Changes">
                </form><br>
            </div>

            <!-- Default Transposh flags -->
            <div id="tlt-transposh-flags" class="tlt-tab-content">
                <h2><?php esc_html_e( 'Transposh Flags', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                <div class="tlt-section"> 
                    <p><?php esc_html_e( 'The images are usually 16x11px, but they appear larger here for improved visibility.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div><br>
                <div class="tlt-section">   
                <?php
                    $flags_folder = 'wp-content/plugins/transposh-translation-filter-for-wordpress/img/flags';
                    $flags_path = ABSPATH . $flags_folder; // Get the absolute path to the flags folder

                    if ( is_dir( $flags_path ) ) {
                        $flags = scandir( $flags_path ); // Retrieve the list of files in the folder

                        echo '<div class="tlt-flags-container">';

                        foreach ( $flags as $flag ) {
                            if ( pathinfo( $flag, PATHINFO_EXTENSION ) === 'png' ) { // Filter only PNG files
                                $flag_code = pathinfo( $flag, PATHINFO_FILENAME );
                                $flag_url = site_url( $flags_folder . '/' . $flag );

                                // Display the flag image and its code
                                echo '<div class="tlt-flag-item">';
                                echo '<img src="' . $flag_url . '"/>';
                                echo '<span>' . esc_html_e( 'Flag Code: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . $flag_code . '</span>';
                                echo '</div>';
                            }
                        }
                        echo '</div>';
                    } else {
                        esc_html_e( 'Flags folder not found.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN );
                    }
                ?>
                </div><br>
            </div>
            
            <!-- Help -->
            <div id="tlt-custom-languages-help" class="tlt-tab-content">
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'How to Add a Language', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'To create a custom language that is not present in Transposh, follow these steps:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <ul>
                        <li><?php esc_html_e( 'Click "Add New Language" to create a new language entry. Initially, this entry is not editable.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></li>
                        <li><?php esc_html_e( 'Click "Modify" to edit the language details. After editing, click "Update" to leave the edit mode.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></li>
                        <li><?php esc_html_e( ' After adding/editing languages, click "Save Changes". You can add multiple languages.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></li>
                    </ul>
                    <p><?php esc_html_e( 'You will find the new language in the Transposh Languages page.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <p><?php esc_html_e( 'Note: Engines translation isn\'t available for the custom languages.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Key and Value Format', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>       
                    <p><?php echo wp_kses( __( 'The <strong>Key</strong> represents the <code>Language Code</code> for the country you want to add. The language code should consist of two or three lowercase alphabetic characters (a-z). Subtags, if present, are separated by hyphens (-). For example, for English, the language code is <code>en</code>". If you need to create "English Singapore", the language code should be <code>en-sg</code>. Please ensure that the key follows this pattern.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array(), 'code' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( 'The <strong>Value</strong> should follow the Transposh format: <code>English Name, Native Name, Flag Code, Local Code</code>, all separated by commas. For example, for French, it would be <code>French,Français,fr,fr_FR</code>". If the local code and the language code are identical, the last parameter, local code, can be omitted.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array(), 'code' => array() ) ); ?></p>
                    <p> <?php esc_html_e( 'Here is a breakdown of the Value format:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <ul>
                        <li><?php echo wp_kses( __( '<strong>English Name:</strong> The language name in English (e.g., French, Chinese, German).', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></li>
                        <li><?php echo wp_kses( __( '<strong>Native Name:</strong> The language name in the native language (e.g., Français, 中文, Deutsch).', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></li>
                        <li><?php echo wp_kses( __( '<strong>Flag Code:</strong> This is the code used by Transposh to retrieve the flag image file. The code is equal to the name of the file. For example, if the French flag file name is <code>fr.png</code>, then the flag code is <code>fr</code>. You can refer to the "Transposh Flags" tab to see the list of available flags. If the flag you need is not present, you can upload your own flag in the following Transpoh folder: ', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array(), 'code' => array() ) ); ?><code>wp-content/plugins/transposh-translation-filter-for-wordpress/img/flags</code><?php esc_html_e( '. Note that after Transposh plugin update, you will have to upload the file again.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></li>
                        <li><?php echo wp_kses( __( '<strong>Local Code:</strong> This is the code used for po file translation. If you use a program like "Poedit" to create a new po translation file based on a pot file, Poedit will generate file names (po and mo) with extensions matching this local code. If you need to manage translation via po files, make sure to choose the correct code for your country. If this code is identical to the key, you can omit it.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></li>
                    </ul>
                </div>
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Some Examples', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2> 
                    <p><?php echo wp_kses( __( 'Singapore English => Key: <code>en-sg</code>, Value: <code>Singapore English,Singapore English,sg,en_SG</code>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( 'Brazilian Portuguese => Key: <code>pt-br</code>, Value: <code>Brazilian Portuguese,Português do Brasil,br,pt_BR</code>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( 'Indian English => Key: <code>en-in</code>, Value: <code>Indian English,English (India),in,en_IN</code>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( 'Algerian French => Key: <code>fr-dz</code>, Value: <code>French (Algeria),Français (Algérie),dz,fr_DZ</code>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                </div><br>
            </div>
        </div>
        <?php
    }
}