<?php
/**
 * Language Menu Shortcodes Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

use Transposh_Language_Toolbox\Base_Utilities;

// Include the transposh file that contains the language constants
require_once WP_PLUGIN_DIR . '/transposh-translation-filter-for-wordpress/core/constants.php';

/**
 * Class to display the list of languages
 */
class Language_Menu_Shortcodes extends Base_Utilities {

    public $transposh_languages;
    private $dynamic_script_handle;
    private $tlt_lang_list_css_added = false;
    private $tlt_flag_drop_css_added = false;
    private $tlt_globe_drop_css_added = false;
    private $tlt_lang_list_css_basic;
    private $tlt_lang_list_css_underline;
    private $tlt_flag_drop_css;
    private $tlt_flag_drop_js;
    private $tlt_globe_drop_css;
    private $tlt_globe_drop_js;

    /**
     * Initialize the shortcodes
     */
    public function init() {
        // In case needed in the shortcodes, set the dynamic script handle
        $this->dynamic_script_handle = 'dynamic-script-' . uniqid();
        // Load transposh languages constants
        $this->transposh_languages = \transposh_consts::$languages;
        // Shortcodes callback
        add_shortcode( 'tlt-lang-list', array( $this, 'menu_lang_list' ) );
        add_shortcode( 'tlt-flag-drop', array( $this, 'menu_flag_drop' ) );
        add_shortcode( 'tlt-globe-drop', array( $this, 'menu_globe_drop' ) );
        // Check if there is possible language filtering for a Transposh language menu
        add_filter( 'do_shortcode_tag', array( $this, 'shortcode_filter_function' ), 10, 4 );
    }

    /**
     * Filter function for modifying the output of the tpe_widget shortcode.
     *
     * @param string $output The shortcode output.
     * @param string $tag The shortcode tag.
     * @param array $atts The shortcode attributes.
     * @param array $m The shortcode regular expression matches.
     * @return string The modified shortcode output.
     */
    public function shortcode_filter_function( $output, $tag, $atts, $m ) {
        if ( 'tpe' === $tag && isset( $atts['widget'] ) ) { // Transposh language menu widget
            $language_to_keep = $this->get_language_to_keep();
            if ( $language_to_keep !== '' ) { 
                return '';
            }
        }
        // Return the $output
        return $output;
    }

    /**
     * Retrieves the language to keep (in language menu) based on the settings and current page context.
     *
     * @return string The language code to keep.
     */
    public function get_language_to_keep() {
        $hide_menus = isset( $this->tlt_options['advanced_settings']['activate_posts_exclude'] ) ? ( $this->tlt_options['advanced_settings']['activate_posts_exclude'] ? true : false ) : false;
        $show_default = isset( $this->tlt_options['advanced_settings']['always_display_default'] ) ? ( $this->tlt_options['advanced_settings']['always_display_default'] ? true : false ) : false;
        $language_to_keep = '';

        if ( $hide_menus && is_single() ) {
            $current_page_id = get_queried_object_id();
            $tp_language = get_post_meta( $current_page_id, 'tp_language', true );

            if ( $tp_language !== '' ) {
                $language_to_keep = $tp_language;
            } else {
                if ( ! $show_default ) {
                    $language_to_keep = $this->default_language;
                }
            }
        }

        return $language_to_keep;
    }

    /**
     * Callback functions for shortcodes
     *
     * @return string The array of viewable_languages
     */
    public function menu_lang_list( $atts ) {
        $atts = shortcode_atts( array(
            'names' => '',
            'load' => '',
        ), $atts );

        // Check the value of the 'load' attribute and set it to null if not 'hover' or 'underline'
        $load_value = $atts['load'];
        if ( $load_value !== 'basic' && $load_value !== 'underline' ) {
            $load_value = null;
        }

        // Get the transposh language names
        $language_names = $this->get_language_names( $atts['names'] );

        // Determine the active language
        $active_language = $this->get_active_language();

        // Generate the HTML structure for the language menu
        $html = $this->render_menu_lang_list( $language_names, $active_language );

        // Set the default CSS
        $this->tlt_lang_list_css_basic ='/* Shortcode tlt-lang-list CSS */
            #tlt-lang-list {
                display: -webkit-inline-box;
                display: -ms-inline-flexbox;
                display: inline-flex;
                -webkit-box-align: center;
                   -ms-flex-align: center;
                      align-items: center;
                -ms-flex-line-pack: center;
                     align-content: center;
                gap: 0.15em;
                -ms-flex-wrap: wrap;
                    flex-wrap: wrap;
                background-color: #ececf3;
                padding: 0.3em 0.5em;
                border-radius: 20px;
            }
            #tlt-lang-list a {
                outline: none;
                text-decoration: none;
            }
            #tlt-lang-list a::focus {
                -webkit-box-shadow: none;
                        box-shadow: none;
            }
            #tlt-lang-list a span {
                line-height: 1;
                padding: 0 0.24em;
                color: #333842;
            }
            #tlt-lang-list a:hover span, #tlt-lang-list a.tr_active span {
                color: #183ad6;
            };';

        $this->tlt_lang_list_css_underline ='/* Shortcode tlt-lang-list hover animation CSS */
            #tlt-lang-list {
                display: -webkit-inline-box;
                display: -ms-inline-flexbox;
                display: inline-flex;
                -webkit-box-align: center;
                   -ms-flex-align: center;
                      align-items: center;
                -ms-flex-line-pack: center;
                     align-content: center;
                gap: 0.15em;
                -ms-flex-wrap: wrap;
                    flex-wrap: wrap;
                background-color: #ececf3;
                padding: 0.3em 0.5em;
                border-radius: 20px;
            }
            #tlt-lang-list a {
                outline: none;
                text-decoration: none;
            }
            #tlt-lang-list a:focus {
                -webkit-box-shadow: none;
                        box-shadow: none;
            }
            #tlt-lang-list a span {
                position: relative;
                line-height: 1;
                padding: 0 0.24em;
                color: #333842;
                -webkit-transition: all 0.2s ease-in-out;
                        transition: all 0.2s ease-in-out;
            }
            #tlt-lang-list a span::after {
                content: "";
                position: absolute;
                width: 0%;
                height: 1px;
                bottom: 0px;
                left: 50%;
                background-color: #333842;
                -webkit-transition: all 0.2s ease-in-out;
                        transition: all 0.2s ease-in-out;
                overflow: hidden;
            }
            #tlt-lang-list a:hover span::after, #tlt-lang-list a.tr_active span::after {
                width: 90%;
                left: 5%;
            }';

        if ( $load_value !== null ) {
            if ( ! $this->tlt_lang_list_css_added ) {
                if ( $load_value === "basic" ) {
                    $html = '<style>' . $this->simple_minify( $this->tlt_lang_list_css_basic ) . '</style>' . $html;
                } else {
                    $html = '<style>' . $this->simple_minify( $this->tlt_lang_list_css_underline ) . '</style>' . $html;
                }
                $this->tlt_lang_list_css_added = true;
            }
        }
        return $html;
    }

    public function menu_flag_drop( $atts ) {
        // Check if the 'load' attribute is present in the $atts array
        $load_css_js = is_array( $atts ) && in_array( 'load', $atts );

        $atts = shortcode_atts( array(
            'names' => '',
        ), $atts );

        // Get the transposh language names
        $language_names = $this->get_language_names( $atts['names'] );

        // Determine the active language
        $active_language = $this->get_active_language();

        // Generate the HTML structure for the language menu
        $html = $this->render_menu_flag_drop( $language_names, $active_language );

        // Set the default CSS and JS
        $this->tlt_flag_drop_css ='/* Shortcode tlt-flag-drop CSS */
            #tlt-flag-drop {
                position: relative;
            }
            #tlt-flag-drop .tlt-flag-drop-img-wrap {
                overflow: hidden;
                border-radius: 50%;
                border: 1px solid #e6e6e6;
                display: inline-block;
            }
            #tlt-flag-drop .tlt-flag-drop-img-wrap img {
                width: 100%;
                height: 100%;
                -o-object-fit: cover;
                   object-fit: cover;
            }
            #tlt-flag-drop .tlt-flag-drop-header {
                display: -webkit-box;
                display: -ms-flexbox;
                display: flex;
                z-index: 10;
                position: relative;
            }
            #tlt-flag-drop .tlt-flag-drop-header .tlt-flag-drop-img-wrap {
                width: 28px;
                height: 28px;
                cursor: pointer;
            }
            #tlt-flag-drop .tlt-flag-drop-down {
                position: absolute;
                top: 21px;
                left: 15px; /* header image + 2 (borders) divided by 2 */
                border: 1px solid #e6e6e6;
                padding: 12px 12px;
                background-color: #ffffff;
                -webkit-transition: all 0.4s ease-in-out;
                transition: all 0.4s ease-in-out;
                -webkit-box-shadow: 0px 8px 28px 0px rgba(0, 0, 0, 0.07);
                        box-shadow: 0px 8px 28px 0px rgba(0, 0, 0, 0.07);
                -webkit-transform: translate3d(-50%, 0, 0);
                        transform: translate3d(-50%, 0, 0);
                opacity: 0;
                z-index: 5;
                visibility: hidden;
            }
            #tlt-flag-drop .tlt-flag-drop-down.tlt-drop-active {
                top: 36px;
                opacity: 1;
                visibility: visible;
            }
            #tlt-flag-drop .tlt-flag-drop-down a {
                display: -webkit-box;
                display: -ms-flexbox;
                display: flex;
                -webkit-box-align: center;
                   -ms-flex-align: center;
                      align-items: center;
                gap: 17px;
                min-width: -webkit-max-content;
                min-width: -moz-max-content;
                min-width: max-content;
                padding: 7px 18px;
                outline: none;
                text-decoration: none;
                color: #333842;
            }
            #tlt-flag-drop .tlt-flag-drop-down a:hover {
                background-color: #f0f0f0;
            }
            #tlt-flag-drop .tlt-flag-drop-down a .tlt-flag-drop-img-wrap {
                width: 18px;
                height: 18px;
            }';

        $this->tlt_flag_drop_js ='/* Shortcode tlt-flag-drop JS */
            (function () {
                var flagContainers = document.querySelectorAll("#tlt-flag-drop");
                for (var i = 0; i < flagContainers.length; i++) {
                    (function () {
                        var flagContainer = flagContainers[i];
                        var flagImage = flagContainer.querySelector(".tlt-flag-drop-header img");
                        var flagDropDown = flagContainer.querySelector(".tlt-flag-drop-down");
                        if (flagImage && flagDropDown) {
                            flagImage.addEventListener("click", function(event) {
                                event.stopPropagation();
                                flagDropDown.classList.toggle("tlt-drop-active");
                
                                if (flagDropDown.classList.contains("tlt-drop-active")) {
                                    document.addEventListener("click", handleOutsideClick);
                                } else {
                                    closeDropdown();
                                }
                            });
                            flagDropDown.addEventListener("click", function(event) {
                                event.stopPropagation();
                            });
                        }
                        function closeDropdown() {
                            flagDropDown.classList.remove("tlt-drop-active");
                            document.removeEventListener("click", handleOutsideClick);
                        }
                        function handleOutsideClick(event) {
                            if (!flagImage.contains(event.target) && !flagDropDown.contains(event.target)) {
                                closeDropdown();
                            }
                        }
                    })();
                }
            })();';

        if ( $load_css_js ) {
            // Enqueue dynamic script if it's not already enqueued
            if ( ! wp_script_is( $this->dynamic_script_handle, 'enqueued' ) ) {
                wp_register_script( $this->dynamic_script_handle, false );
                wp_enqueue_script( $this->dynamic_script_handle, '', array(), false, true );
            }
            if ( ! $this->tlt_flag_drop_css_added ) {
                $html = '<style>' . $this->simple_minify( $this->tlt_flag_drop_css ) . '</style>' . $html;
                wp_add_inline_script( $this->dynamic_script_handle, $this->simple_minify( $this->tlt_flag_drop_js ) );
                $this->tlt_flag_drop_css_added = true;
            }
        }

        return $html;
    }

    public function menu_globe_drop( $atts ) {
        // Check if the 'load' attribute is present in the $atts array
        $load_css_js = is_array( $atts ) && in_array( 'load', $atts );

        $atts = shortcode_atts( array(
            'names' => '',
        ), $atts );

        // Get the transposh language names
        $language_names = $this->get_language_names( $atts['names'] );

        // Determine the active language
        $active_language = $this->get_active_language();

        // Generate the HTML structure for the language menu
        $html = $this->render_menu_globe_drop( $language_names, $active_language );

        // Set the default CSS and JS
        $this->tlt_globe_drop_css ='/* Shortcode tlt-globe-drop CSS */
            #tlt-globe-drop {
                position: relative;
            }
            #tlt-globe-drop .tlt-globe-drop-header {
                display: -webkit-box;
                display: -ms-flexbox;
                display: flex;
                -webkit-box-pack: center;
                    -ms-flex-pack: center;
                        justify-content: center;
                -webkit-box-align: center;
                    -ms-flex-align: center;
                        align-items: center;
                width: 2em;
                height: 2em;
                border-radius: 50%;
                position: relative;
                z-index: 10;
                font-size: 1.2rem;
            }
            #tlt-globe-drop .tlt-globe-drop-header:hover {
                background-color: #f0f0f0;
                cursor: pointer;
            }
            #tlt-globe-drop .tlt-globe-drop-header svg {
                display: block;
                width: 1em;
                height: 1em;
            }
            #tlt-globe-drop .tlt-globe-drop-down {
                position: absolute;
                top: 21px;
                left: 1em; /* tlt-globe-drop-header width divided by 2 */
                border: 1px solid #e6e6e6;
                padding: 12px 12px;
                background-color: #ffffff;
                -webkit-transition: all 0.4s ease-in-out;
                transition: all 0.4s ease-in-out;
                -webkit-box-shadow: 0px 8px 28px 0px rgba(0, 0, 0, 0.07);
                        box-shadow: 0px 8px 28px 0px rgba(0, 0, 0, 0.07);
                -webkit-transform: translate3d(-50%, 0, 0);
                        transform: translate3d(-50%, 0, 0);
                opacity: 0;
                z-index: 5;
                visibility: hidden;
            }
            #tlt-globe-drop .tlt-globe-drop-down.tlt-drop-active {
                top: 45px;
                opacity: 1;
                visibility: visible;
            }
            #tlt-globe-drop .tlt-globe-drop-down a {
                display: -webkit-box;
                display: -ms-flexbox;
                display: flex;
                -webkit-box-align: center;
                    -ms-flex-align: center;
                        align-items: center;
                gap: 17px;
                min-width: -webkit-max-content;
                min-width: -moz-max-content;
                min-width: max-content;
                padding: 7px 24px 7px 18px;
                outline: none;
                text-decoration: none;
                color: #333842;
            }
            #tlt-globe-drop .tlt-globe-drop-down a:hover {
                background-color: #f0f0f0;
            }
            #tlt-globe-drop .tlt-globe-drop-down a.tr_active {
                position: relative;
            }
            #tlt-globe-drop .tlt-globe-drop-down a.tr_active svg {
                position: absolute;
                right: 0;
                top: 50%;
                -webkit-transform: translateY(-50%);
                        transform: translateY(-50%);
                width: 1em;
                height: 1em;
            }';

        $this->tlt_globe_drop_js ='/* Shortcode tlt-globe-drop JS */
            (function () {
                var flagContainers = document.querySelectorAll("#tlt-globe-drop");
                for (var i = 0; i < flagContainers.length; i++) {
                    (function () {
                        var flagContainer = flagContainers[i];
                        var globeDropHeader = flagContainer.querySelector(".tlt-globe-drop-header");
                        var globeDropDown = flagContainer.querySelector(".tlt-globe-drop-down");
                        if (globeDropHeader && globeDropDown) {
                            globeDropHeader.addEventListener("click", function(event) {
                                event.stopPropagation();
                                globeDropDown.classList.toggle("tlt-drop-active");
                    
                                if (globeDropDown.classList.contains("tlt-drop-active")) {
                                    document.addEventListener("click", handleOutsideClick);
                                } else {
                                    closeDropdown();
                                }
                            });
                            globeDropDown.addEventListener("click", function(event) {
                                event.stopPropagation();
                            });
                        }
                        function closeDropdown() {
                            globeDropDown.classList.remove("tlt-drop-active");
                            document.removeEventListener("click", handleOutsideClick);
                        }
                        function handleOutsideClick(event) {
                            if (!globeDropHeader.contains(event.target) && !globeDropDown.contains(event.target)) {
                                closeDropdown();
                            }
                        }
                    })();
                }
            })();';

        if ( $load_css_js ) {
            // Enqueue dynamic script if it's not already enqueued
            if ( ! wp_script_is( $this->dynamic_script_handle, 'enqueued' ) ) {
                wp_register_script( $this->dynamic_script_handle, false );
                wp_enqueue_script( $this->dynamic_script_handle, '', array(), false, true );
            }
            if ( ! $this->tlt_globe_drop_css_added ) {
                $html = '<style>' . $this->simple_minify( $this->tlt_globe_drop_css ) . '</style>' . $html;
                wp_add_inline_script( $this->dynamic_script_handle, $this->simple_minify( $this->tlt_globe_drop_js ) );
                $this->tlt_globe_drop_css_added = true;
            }
        }

        return $html;        
    }

    /**
     * Get the language names based on the shortcode argument or use the default Transposh names
     *
     * @param string $names_arg The names argument from the shortcode
     * @return array The language names
     */
    private function get_language_names( $names_arg ) {
        $viewable_languages_array = explode( ',', $this->viewable_languages );
        $custom_names = explode( ',', $names_arg );

        // Use custom names if the number of custom names matches the number of viewable languages
        if ( count( $custom_names ) === count( $viewable_languages_array ) ) {
            return array_combine( $viewable_languages_array, $custom_names );
        }

        // Use default Transposh names
        return array_map( function( $language_data ) {
            $language_parts = explode( ',', $language_data );
            return isset( $language_parts[1] ) ? $language_parts[1] : '';
        }, $this->transposh_languages );
    }

    /**
     * Get the url of the target language based on the current URL
     *
     * @param string $lang The target language code
     * @return string The target url
     */
    private function get_target_url( $lang ) {
        $home = home_url('/');
        //$current_url = home_url( add_query_arg( $_GET, $GLOBALS['wp']->request ) );
        $current_url = add_query_arg( $_SERVER['QUERY_STRING'], '', home_url( $GLOBALS['wp']->request ) );
        $current_url = htmlentities( $current_url );

        $transposh_enable_permalinks = isset( $this->tp_options['enable_permalinks'] ) ? $this->tp_options['enable_permalinks'] : 1;
        $url = \transposh_utils::rewrite_url_lang_param( $current_url, $home, $transposh_enable_permalinks, $lang, false );

        // Check if we need to remove the language code in case target language is default language
        if ( $lang === $this->default_language ) {
            $default_lang_slug = '/' . $this->default_language;
            $url = str_replace( $default_lang_slug, '', $url );
        }
        
        // Clean the url (in case last "\" is missing)
        $parts = parse_url( $url );
        if ( isset( $parts['path'] ) && substr( $parts['path'], -1) !== '/' ) {
            $modified_url = $parts['scheme'] . '://' . $parts['host'] . $parts['path'] . '/';
            if ( isset( $parts['query'] ) ) {
                $modified_url .= '?' . $parts['query'];
            }
            $url = $modified_url;
        }

        return $url;
    }

    /**
     * Render the HTML structure for the language menu
     *
     * @param string $active_language The active language code
     * @param string $current_url The current page URL
     * @return string The rendered HTML
     */
    private function render_menu_lang_list( $language_names, $active_language ) {
        $id_main = 'tlt-lang-list';
        $class_items = 'tlt-lang-list-item';

        ob_start();
        ?>
        <div id="<?php echo $id_main ?>" class="no_translate">
            <?php
            $viewable_languages_array = explode( ',', $this->viewable_languages );

            // check if we have to hide a menu (maybe)
            $language_to_keep = $this->get_language_to_keep();
            
            foreach ( $viewable_languages_array as $index => $language_code ) {
                $language_code = trim( $language_code );
                $is_active_language = ( $language_code === $active_language );
                $language_name = isset( $language_names[$language_code] ) ? trim( $language_names[$language_code] ) : '';
                $class_unavailable = ' tlt-link-unavailable'; // not used but this might be helpfull for someone
                $style_unavailable = 'style="opacity:0.5;pointer-events:none;"';
                if ( ( $language_to_keep === '' ) || ( $language_to_keep === $language_code ) ) {
                    $class_unavailable = '';
                    $style_unavailable = '';
                }
                ?>
                <a href="<?php echo $this->get_target_url( $language_code ); ?>" class="<?php echo $class_items . $class_unavailable; if ( $is_active_language ) echo ' tr_active';?>" <?php echo $style_unavailable; ?>>
                    <span title="<?php echo esc_attr( $language_name ); ?>"><?php echo esc_attr( $language_name ); ?></span>
                </a>
                <?php
            }
            ?>
        </div>
        <?php
        return ob_get_clean();
    }

    /**
     * Render the HTML structure for the language menu
     *
     * @param string $active_language The active language code
     * @param string $current_url The current page URL
     * @return string The rendered HTML
     */
    private function render_menu_flag_drop( $language_names, $active_language ) {
        $id_main = 'tlt-flag-drop';
        $class_items = 'tlt-flag-drop-item';
        $flags_folder = 'wp-content/plugins/transposh-translation-filter-for-wordpress/img/flags/';
        $flags_path = site_url( $flags_folder ); // Get the path to the flags folder

        ob_start();
        ?>
        <div id="<?php echo $id_main ?>" class="no_translate">
            <div class="tlt-flag-drop-header">
                <?php 
                $flag_code = isset( $this->transposh_languages[$active_language] ) ? explode( ',', $this->transposh_languages[$active_language] )[2] ?? null : null;
                $flag_url = $flags_path . $flag_code . '.png';
                ?>
                <span class="tlt-flag-drop-img-wrap"><img src="<?php echo $flag_url; ?>"/></span>
            </div>
            <div class="tlt-flag-drop-down">
                <?php
                $viewable_languages_array = explode( ',', $this->viewable_languages );

                // check if we have to hide a menu (maybe)
                $language_to_keep = $this->get_language_to_keep();
                
                foreach ( $viewable_languages_array as $index => $language_code ) {
                    $language_code = trim( $language_code );
                    $is_active_language = ( $language_code === $active_language );
                    $language_name = isset( $language_names[$language_code] ) ? trim( $language_names[$language_code] ) : '';
                    $flag_code = isset( $this->transposh_languages[$language_code] ) ? explode( ',', $this->transposh_languages[$language_code] )[2] ?? null : null;
                    $flag_url = $flags_path . $flag_code . '.png';
                    $class_unavailable = ' tlt-link-unavailable'; // not used but this might be helpfull for someone
                    $style_unavailable = 'style="opacity:0.5;pointer-events:none;"';
                    if ( ( $language_to_keep === '' ) || ( $language_to_keep === $language_code ) ) {
                        $class_unavailable = '';
                        $style_unavailable = '';
                    }
                    ?>
                    <a href="<?php echo $this->get_target_url( $language_code ); ?>" class="<?php echo $class_items . $class_unavailable; if ( $is_active_language ) echo ' tr_active';?>" <?php echo $style_unavailable; ?>>
                        <span class="tlt-flag-drop-img-wrap"><img src="<?php echo $flag_url; ?>"/></span>
                        <span title="<?php echo esc_attr( $language_name ); ?>"><?php echo esc_attr( $language_name ); ?></span>
                    </a>
                    <?php
                }
                ?>
            </div>
        </div>
        <?php
        return ob_get_clean();
    }

    /**
     * Render the HTML structure for the language menu
     *
     * @param string $active_language The active language code
     * @param string $current_url The current page URL
     * @return string The rendered HTML
     */
    private function render_menu_globe_drop( $language_names, $active_language ) {
        $id_main = 'tlt-globe-drop';
        $class_items = 'tlt-globe-drop-item';

        ob_start();
        ?>
        <div id="<?php echo $id_main ?>" class="no_translate">
            <div class="tlt-globe-drop-header">
                <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 16 16" fill="currentColor" aria-hidden="false" role="presentation" focusable="false">
                    <path d="M8 .25a7.77 7.77 0 0 1 7.75 7.78 7.75 7.75 0 0 1-7.52 7.72h-.25A7.75 7.75 0 0 1 .25 8.24v-.25A7.75 7.75 0 0 1 8 .25zm1.95 8.5h-3.9c.15 2.9 1.17 5.34 1.88 5.5H8c.68 0 1.72-2.37 1.93-5.23zm4.26 0h-2.76c-.09 1.96-.53 3.78-1.18 5.08A6.26 6.26 0 0 0 14.17 9zm-9.67 0H1.8a6.26 6.26 0 0 0 3.94 5.08 12.59 12.59 0 0 1-1.16-4.7l-.03-.38zm1.2-6.58-.12.05a6.26 6.26 0 0 0-3.83 5.03h2.75c.09-1.83.48-3.54 1.06-4.81zm2.25-.42c-.7 0-1.78 2.51-1.94 5.5h3.9c-.15-2.9-1.18-5.34-1.89-5.5h-.07zm2.28.43.03.05a12.95 12.95 0 0 1 1.15 5.02h2.75a6.28 6.28 0 0 0-3.93-5.07z"></path>
                </svg>
            </div>
            <div class="tlt-globe-drop-down">
                <?php
                $viewable_languages_array = explode( ',', $this->viewable_languages );

                // check if we have to hide a menu (maybe)
                $language_to_keep = $this->get_language_to_keep();
                
                foreach ( $viewable_languages_array as $index => $language_code ) {
                    $language_code = trim( $language_code );
                    $is_active_language = ( $language_code === $active_language );
                    $language_name = isset( $language_names[$language_code] ) ? trim( $language_names[$language_code] ) : '';
                    $class_unavailable = ' tlt-link-unavailable'; // not used but this might be helpfull for someone
                    $style_unavailable = 'style="opacity:0.5;pointer-events:none;"';
                    if ( ( $language_to_keep === '' ) || ( $language_to_keep === $language_code ) ) {
                        $class_unavailable = '';
                        $style_unavailable = '';
                    }
                    ?>
                    <a href="<?php echo $this->get_target_url( $language_code ); ?>" class="<?php echo $class_items . $class_unavailable; if ( $is_active_language ) echo ' tr_active';?>" <?php echo $style_unavailable; ?>>
                        <span title="<?php echo esc_attr( $language_name ); ?>"><?php echo esc_attr( $language_name ); ?></span>
                        <?php if ( $is_active_language ) {?>
                            <svg xmlns="http://www.w3.org/2000/svg" viewBox="0 0 128 128" fill="currentColor"><path d="M56.33 100a4 4 0 0 1-2.82-1.16L20.68 66.12a4 4 0 1 1 5.64-5.65l29.57 29.46 45.42-60.33a4 4 0 1 1 6.38 4.8l-48.17 64a4 4 0 0 1-2.91 1.6z"></path></svg>
                        <?php } ?> 
                    </a>
                    <?php
                }
                ?>
            </div>
        </div>
        <?php
        return ob_get_clean();
    }

    /**
     * Render the shortcode explanation content.
     */
    public function render_shortcode_explanation() {
        ?>
        <div class="tlt-wrap">

            <div class="tlt-menu">
                <a href="#" class="tlt-menu-item active" data-tab="tlt-custom-shortcodes"><?php esc_html_e( 'Custom Shortcodes', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-transposh-default-shortcodes"><?php esc_html_e( 'Transposh Default Shortcodes', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
                <span class="tlt-menu-divider">|</span>
                <a href="#" class="tlt-menu-item" data-tab="tlt-shortcode-usage"><?php esc_html_e( 'Shortcode Usage', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></a>
            </div>

            <!-- Custom shortcodes-->
            <div id="tlt-custom-shortcodes" class="tlt-tab-content active">
                <div class="tlt-section tlt-intro">
                    <h2><?php esc_html_e( 'Custom Shortcodes', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'Here\'s a list of custom shortcodes. The right side displays the result with custom CSS/JS applied to the languages of your website. Click on a row to view the associated CSS and JS. For detailed instructions on using a shortcode and customizing its appearance, refer to the \'Shortcode Usage\' section.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <h2><?php esc_html_e( 'Shortcode Arguments', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php echo wp_kses( __( 'To replace the default Transposh language names, use the "names" argument, separated by commas: <code>[tlt-lang-list names="eng,fr,esp"]</code>. The names should respect the number and order in which they appear on your website.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( 'We recommend copying and modifying the associated CSS and JS in your child theme, ensuring that the JS is loaded in the footer. If you prefer an automatic approach that includes the necessary CSS and JS inline, you can use the "load" attribute. Example: <code>[tlt-flag-drop load]</code>. Note that <code>[tlt-lang-list]</code> requires specifying which CSS you wish to load (refer to the down list). Other shortcodes only need the "load" attribute.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                    <p><?php echo wp_kses( __( 'It is possible to combine attributes: <code>[tlt-globe-drop names="eng,fr" load]</code>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                    <h2><?php esc_html_e( 'Shortcode List', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p class="tlt-important"><?php esc_html_e( 'Click on a row to reveal the CSS/JS.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
                <!-- Shortcode [tlt-lang-list] -->
                <div class="tlt-section tlt-row tlt-clickable">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tlt-lang-list]</pre><br>
                        <pre class="tlt-code-block">[tlt-lang-list load="basic"]</pre><br>
                        <pre class="tlt-code-block">[tlt-lang-list load="underline"]</pre><br>
                        <pre class="tlt-code-block">[tlt-lang-list names="Language 1, Language 2, ..."]</pre>
                    </div>
                    <div class="tlt-right">
                        <div><?php esc_html_e( 'Basic hover effect:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <?php 
                            ob_start();
                            echo $this->menu_lang_list( array() );
                            $shortcode_output = ob_get_clean();
                            // Replace links with "#"
                            $shortcode_output = preg_replace('/<a\b[^>]*>/', '<a href="#">', $shortcode_output);
                            echo $shortcode_output;
                        ?>
                        <br><br>
                        <div><?php esc_html_e( 'Hover underline effect:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <div class="tlt-hover"><?php 
                            ob_start();
                            echo $this->menu_lang_list( array() );
                            $shortcode_output = ob_get_clean();
                            // Replace links with "#"
                            $shortcode_output = preg_replace('/<a\b[^>]*>/', '<a href="#">', $shortcode_output);
                            echo $shortcode_output;
                        ?></div> 
                    </div>
                </div>
                <div class="tlt-section tlt-content tlt-hidden">
                    <h3 class="tlt-top-title"><?php esc_html_e( 'CSS for basic hover effect:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h3>
                    <div class="tlt-code-block-container">
                        <a class="tlt-copy-css-link" title="<?php esc_html_e( 'Copy CSS', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>"></a>
                        <div class="tlt-copy-message"><?php esc_html_e( 'CSS Copied!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <pre class="tlt-code-block"><?php echo $this->simple_minify( $this->tlt_lang_list_css_basic, 12 ); ?></pre>
                    </div>
                    <br><br>
                    <h3 class="tlt-top-title"><?php esc_html_e( 'CSS for hover underline effect:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h3>
                    <div class="tlt-code-block-container">
                        <a class="tlt-copy-css-link" title="<?php esc_html_e( 'Copy CSS', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>"></a>
                        <div class="tlt-copy-message"><?php esc_html_e( 'CSS Copied!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <pre class="tlt-code-block"><?php echo $this->simple_minify( $this->tlt_lang_list_css_underline, 12 ); ?></pre>
                    </div>
                </div>
                <!-- Shortcode [tlt-flag-drop] -->
                <div class="tlt-section tlt-row tlt-clickable">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tlt-flag-drop]</pre><br>
                        <pre class="tlt-code-block">[tlt-flag-drop load]</pre><br>
                        <pre class="tlt-code-block">[tlt-flag-drop names="Language 1, Language 2, ..."]</pre>
                    </div>
                    <div class="tlt-right">
                        <div><?php esc_html_e( 'Click to reveal:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <?php 
                            ob_start();
                            echo $this->menu_flag_drop( array() );
                            $shortcode_output = ob_get_clean();
                            // Replace links with "#"
                            $shortcode_output = preg_replace('/<a\b[^>]*>/', '<a href="#">', $shortcode_output);
                            echo $shortcode_output;
                        ?>
                    </div>
                </div>
                <div class="tlt-section tlt-content tlt-hidden">
                    <h3 class="tlt-top-title"><?php esc_html_e( 'CSS:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h3>
                    <div class="tlt-code-block-container">
                        <a class="tlt-copy-css-link" title="<?php esc_html_e( 'Copy CSS', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>"></a>
                        <div class="tlt-copy-message"><?php esc_html_e( 'CSS Copied!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <pre class="tlt-code-block"><?php echo $this->simple_minify( $this->tlt_flag_drop_css, 12 ); ?></pre>
                    </div>
                    <br><br>
                    <h3 class="tlt-top-title"><?php esc_html_e( 'JS:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h3>
                    <div class="tlt-code-block-container">
                        <a class="tlt-copy-css-link" title="<?php esc_html_e( 'Copy JS', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>"></a>
                        <div class="tlt-copy-message"><?php esc_html_e( 'JS Copied!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <pre class="tlt-code-block"><?php echo $this->simple_minify( $this->tlt_flag_drop_js, 12 ); ?></pre>
                    </div>
                </div>
                <!-- Shortcode [tlt-globe-drop] -->
                <div class="tlt-section tlt-row tlt-clickable">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tlt-globe-drop]</pre><br>
                        <pre class="tlt-code-block">[tlt-globe-drop load]</pre><br>
                        <pre class="tlt-code-block">[tlt-globe-drop names="Language 1, Language 2, ..."]</pre>
                    </div>
                    <div class="tlt-right">
                        <div><?php esc_html_e( 'Click to reveal:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <?php 
                            ob_start();
                            echo $this->menu_globe_drop( array() );
                            $shortcode_output = ob_get_clean();
                            // Replace links with "#"
                            $shortcode_output = preg_replace( '/<a\b[^>]*>/', '<a href="#">', $shortcode_output );
                            $shortcode_output = preg_replace( '/<a\b[^>]*>/', '<a href="#" class="tr_active">', $shortcode_output, 1 );
                            echo $shortcode_output;
                        ?>
                    </div>
                </div>
                <div class="tlt-section tlt-content tlt-hidden">
                    <h3 class="tlt-top-title"><?php esc_html_e( 'CSS:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h3>
                    <div class="tlt-code-block-container">
                        <a class="tlt-copy-css-link" title="<?php esc_html_e( 'Copy CSS', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>"></a>
                        <div class="tlt-copy-message"><?php esc_html_e( 'CSS Copied!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <pre class="tlt-code-block"><?php echo $this->simple_minify( $this->tlt_globe_drop_css, 12 ); ?></pre>
                    </div>
                    <br><br>
                    <h3 class="tlt-top-title"><?php esc_html_e( 'JS:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h3>
                    <div class="tlt-code-block-container">
                        <a class="tlt-copy-css-link" title="<?php esc_html_e( 'Copy JS', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?>"></a>
                        <div class="tlt-copy-message"><?php esc_html_e( 'JS Copied!', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></div>
                        <pre class="tlt-code-block"><?php echo $this->simple_minify( $this->tlt_globe_drop_js, 12 ); ?></pre>
                    </div>
                </div>

            </div>

            <!-- Transposh default shortcodes -->
            <div id="tlt-transposh-default-shortcodes" class="tlt-tab-content">
                <div class="tlt-section tlt-intro">
                    <h2><?php esc_html_e( 'Transposh Default Shortcodes', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'Here is a list of the default Transposh shortcodes and their corresponding results for your website\'s languages.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <p><?php echo wp_kses( __( 'Please note that if you\'re using a custom language, the associated flag may not display correctly with these shortcodes: <code>[tpe widget="flags/tpw_flags_css.php"]</code>, <code>[tpe widget="flagslist/tpw_list_with_flags_css.php"]</code>, and <code>[tpe widget="select2/tpw_select2.php"]</code>, as they use their own flags picture.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                    <br>
                </div>
                <div class="tlt-section tlt-row">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tpe widget="default/tpw_default.php"]</pre><br>
                    </div>
                    <div class="tlt-right">
                        <?php 
                            $tpw_shortcode_output = do_shortcode( '[tpe widget="default/tpw_default.php"]' );
                            $modified_shortcode_output = preg_replace( '/<select\b[^>]*\bonchange="[^"]*"/', '<select>', $tpw_shortcode_output );
                            echo $modified_shortcode_output;
                        ?>    
                    </div>
                </div>
                <div class="tlt-section tlt-row" style="z-index:5;">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tpe widget="dropdown/tpw_image_dropdown.php"]</pre><br>
                    </div>
                    <div class="tlt-right">
                    <?php 
                        $tpw_shortcode_output = do_shortcode( '[tpe widget="dropdown/tpw_image_dropdown.php"]' );
                        $modified_shortcode_output = str_replace( '<li><a', '<li onmouseover="this.style.backgroundColor=\'#d0c9af\';" onmouseout="this.style.backgroundColor=\'\';"><a style="pointer-events: none; cursor: default;"', $tpw_shortcode_output );
                        echo $modified_shortcode_output;
                    ?>
                    </div>
                </div>
                <div class="tlt-section tlt-row">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tpe widget="flags/tpw_flags.php"]</pre><br>
                    </div>
                    <div class="tlt-right">
                        <?php
                            $tpw_shortcode_output = do_shortcode( '[tpe widget="flags/tpw_flags.php"]' );
                            $modified_shortcode_output = str_replace( 'href=', 'data-href=', $tpw_shortcode_output );
                            echo $modified_shortcode_output;
                        ?>
                    </div>
                </div>
                <div class="tlt-section tlt-row">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tpe widget="flags/tpw_flags_css.php"]</pre><br>
                    </div>
                    <div class="tlt-right">
                        <?php
                            $tpw_shortcode_output = do_shortcode( '[tpe widget="flags/tpw_flags_css.php"]' );
                            $modified_shortcode_output = str_replace( 'href=', 'data-href=', $tpw_shortcode_output );
                            echo $modified_shortcode_output;
                        ?>
                    </div>
                </div>
                <div class="tlt-section tlt-row">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tpe widget="flagslist/tpw_list_with_flags.php"]</pre><br>
                    </div>
                    <div class="tlt-right">
                        <?php
                            $tpw_shortcode_output = do_shortcode( '[tpe widget="flagslist/tpw_list_with_flags.php"]' );
                            $modified_shortcode_output = str_replace( 'href=', 'data-href=', $tpw_shortcode_output );
                            echo $modified_shortcode_output;
                        ?>
                    </div>
                </div>
                <div class="tlt-section tlt-row">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tpe widget="flagslist/tpw_list_with_flags_css.php"]</pre><br>
                    </div>
                    <div class="tlt-right">
                        <?php
                            $tpw_shortcode_output = do_shortcode( '[tpe widget="flagslist/tpw_list_with_flags_css.php"]' );
                            $modified_shortcode_output = str_replace( 'href=', 'data-href=', $tpw_shortcode_output );
                            echo $modified_shortcode_output;
                        ?>
                    </div>
                </div>
                <div class="tlt-section tlt-row">
                    <div class="tlt-left">
                        <pre class="tlt-code-block">[tpe widget="select2/tpw_select2.php"]</pre><br>
                    </div>
                    <div class="tlt-right">
                        <?php
                            $tpw_shortcode_output = do_shortcode( '[tpe widget="select2/tpw_select2.php"]' );
                            $modified_shortcode_output = preg_replace( '/(<select[^>]+)onchange="[^"]+"/i', '$1', $tpw_shortcode_output );
                            echo $modified_shortcode_output;
                        ?>
                    </div>
                </div>
            </div>
            
            <!-- Usage explanation -->
            <div id="tlt-shortcode-usage" class="tlt-tab-content">
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Shortcodes Usage', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'Simply add the shortcode to any page or post where you want to display the language list.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Customizing With CSS and JS', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php esc_html_e( 'You have several options to add custom code to your WordPress site:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <ul>
                        <li><?php echo wp_kses( __( '<strong>Child Theme</strong>: If you are using a child theme (recommended), you can add the CSS and JS code to your child theme\'s dedicated CSS and JS files. A child theme ensures that your customizations remain intact even after updating the parent theme.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></li>
                        <li><?php echo wp_kses( __( '<strong>Theme Customizer</strong>: Many themes include a built-in customizer that simplifies the process. Locate the "Customize" option in your WordPress dashboard\'s "Appearance" menu. Inside the theme customizer, you\'ll likely find dedicated sections for adding custom CSS and JS. Paste your code into the provided fields.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></li>
                        <li><?php echo wp_kses( __( '<strong>Code Snippet Plugin</strong>: If you prefer not to modify theme files directly, code snippet plugins offer an efficient solution. Install and activate a code snippet plugin, create a new snippet, and insert your CSS or JS code there.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'strong' => array() ) ); ?></li>
                    </ul>
                    <p><?php esc_html_e( 'Feel free to modify the provided CSS and JS to match your prefered visual style or align with your theme\'s color scheme.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <p><?php esc_html_e( 'The JS should be loaded in the footer.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                    <p><?php esc_html_e( 'Keep in mind that if you\'re using optimization or caching plugin, you may need to clear the cache after applying CSS or JS changes to ensure that your updates are reflected on the frontend.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></p>
                </div>
                <div class="tlt-section">
                    <h2><?php esc_html_e( 'Front-End Translation', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ); ?></h2>
                    <p><?php echo wp_kses( __( 'To enable front-end translation, simply add "?tpedit=1" to the URL of the page. This will activate the translation mode. For instance, if your page URL is <code>https://example.com/fr/my-page/</code>, you can activate the translation mode by visiting <code>https://example.com/fr/my-page/?tpedit=1</code>.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ), array ( 'code' => array() ) ); ?></p>
                </div>
            </div>

        </div>
        <?php
    } 
}
