<?php
/**
 * Transposh Language Toolbox Main Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

use Transposh_Language_Toolbox\About;
use Transposh_Language_Toolbox\Translation_Editor;
use Transposh_Language_Toolbox\Omahoung_Update_Checker;
use Transposh_Language_Toolbox\Language_Menu_Shortcodes;

/**
 * Main class for the Transposh Translation Editor plugin.
 */
class Language_Toolbox_Main {
    
    private $translation_editor;
    private $about;
    private $shortcode;
    private $custom_languages;
    private $advanced_settings;
    private $dynamic_css_handle;
    private $tlt_cond_display_css_added = false;
    private $dynamic_script_handle;
    private $tlt_remove_empty_tags_script_added = false;
	
    /**
	 * Initialize the plugin.
	 */
	public function init() {
        // For Admin and Front
        $this->advanced_settings = new Advanced_Settings();
        $this->shortcode = new Language_Menu_Shortcodes();
        $this->shortcode->init();

        // Admin only
        if ( is_admin() ) {
            // Enqueue necessary scripts
            add_action( 'admin_enqueue_scripts', array( $this, 'enqueue_admin_scripts' ) );
            // Instantiate the update checker
            new Omahoung_Update_Checker();
            // Add submenu page under Transposh menu.
            add_action( 'admin_menu', array( $this, 'add_submenu_page' ), 9999 );
            // Add settings link on plugins page
            $plugin_path = plugin_dir_path( dirname( __FILE__, 1 ) );
            $plugin_name = basename( $plugin_path ) . '/' . basename( $plugin_path ) . '.php';
            add_filter( 'plugin_action_links_' . $plugin_name, array( $this, 'settings_link' ) );
            // Register the AJAX action callbacks
            add_action( 'wp_ajax_update_translation', array( __NAMESPACE__ . '\Translation_Editor', 'update_translation_callback' ) );
            add_action( 'wp_ajax_export_csv', array( __NAMESPACE__ . '\Translation_Editor', 'export_csv_callback' ) );
            // Add screen options
            add_action( 'load-transposh_page_transposh-language-toolbox', array( $this, 'add_screen_options' ) );
            //Initialize the advanced settings
            add_action( 'admin_init', array( $this, 'advanced_settings_init' ) );
            // Check if we need to redirect Translators
            add_action( 'admin_init', array( $this->advanced_settings, 'restrict_admin_access' ) );
            // Instantiate the About page
            $this->about = new About();
        }

        // Front only
        if ( ! is_admin() ) {
            // In case needed, set the dynamic CSS/Script handles
            $this->dynamic_css_handle = 'dynamic-css-' . uniqid();
            $this->dynamic_script_handle = 'dynamic-script-' . uniqid();
            add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_front_scripts' ) );
            $this->advanced_settings->load_excerpts_lengths();
            $this->advanced_settings->load_posts_exclusion();
            $this->advanced_settings->search_exclude_sidebar();
            $this->advanced_settings->hide_admin_top_bar();
            $this->advanced_settings->front_translator_role();
            $this->advanced_settings->seo_integration();
        }
	}

    /**
     * Enqueue front-end scripts and styles.
     */
    public function enqueue_front_scripts() {
        // Check if the remove empty html elements option is enabled
        if ( $this->advanced_settings->is_remove_empty_html_elements_enabled() ) {
            // Enqueue dynamic script if it's not already enqueued
            if ( ! wp_script_is( $this->dynamic_script_handle, 'enqueued' ) ) {
                wp_register_script( $this->dynamic_script_handle, false );
                wp_enqueue_script( $this->dynamic_script_handle, '', array(), false, true );
            }
            if ( ! $this->tlt_remove_empty_tags_script_added ) {
                $js = $this->advanced_settings->get_remove_empty_html_elements_js();
                wp_add_inline_script( $this->dynamic_script_handle, $js );
                $this->tlt_remove_empty_tags_script_added = true;
            }
        }

        // Check if the conditional display is enabled
        if ( $this->advanced_settings->is_conditional_display_enabled() ) {
            // Enqueue dynamic style if it's not already enqueued
            if ( ! wp_style_is( $this->dynamic_css_handle, 'enqueued' ) ) {
                wp_register_style( $this->dynamic_css_handle, false );
                wp_enqueue_style( $this->dynamic_css_handle );
            }
            if ( ! $this->tlt_cond_display_css_added ) {
                $css = $this->advanced_settings->get_conditional_display_css();
                wp_add_inline_style( $this->dynamic_css_handle, $css );
                $this->tlt_cond_display_css_added = true;
            }
        }
    }

    /**
     * Enqueue admin scripts and styles.
     */
    public function enqueue_admin_scripts() {
        // Check if the current screen is our plugin page
        if ( isset( $_GET['page'] ) && $_GET['page'] === 'transposh-language-toolbox' ) {
            // All admin CSS
            wp_enqueue_style( 'transposh-language-toolbox-css-main', plugin_dir_url( __FILE__ ) . '../assets/css/tlt-main.min.css' );
            
            // Language Shortcodes JS (always loaded as it contains common script )
            wp_enqueue_script( 'transposh-language-toolbox-js-main', plugin_dir_url( __FILE__ ) . '../assets/js/tlt-main-min.js', array(), '1.0', true );

            // Translation Editor JS
            if ( ! isset( $_GET['tab'] ) || $_GET['tab'] === 'edit_translations' ) {
                wp_enqueue_script( 'transposh-language-toolbox-js-editor', plugin_dir_url( __FILE__ ) . '../assets/js/tlt-editable-fields-min.js', array(), '1.0', true );
                wp_localize_script( 'transposh-language-toolbox-js-editor', 'tltTranslationEditorData', array(
                    'tltTranslationEditorNonce' => wp_create_nonce( 'tlt_translation_editor_action' ),
                    'updateSuccessful' => esc_html__( 'Update successful', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'updateFailed' => esc_html__( 'Update failed', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    //'ajax_url'  => admin_url( 'admin-ajax.php' ),
                ) );
                wp_enqueue_script( 'transposh-language-toolbox-js-export-csv', plugin_dir_url( __FILE__ ) . '../assets/js/tlt-csv-export-min.js', array(), '1.0', true );
                wp_localize_script( 'transposh-language-toolbox-js-export-csv', 'tltEditorExportCsvData', array(
                    'tltEditorExportCsvNonce' => wp_create_nonce( 'tlt_editor_export_csv_nonce' ),
                    'exportSuccessful' => esc_html__( 'Export successful', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'exportFailed' => esc_html__( 'Export failed', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                ) );
            }

            // Language Shortcodes JS
            if ( isset( $_GET['tab'] ) && $_GET['tab'] === 'language_shortcodes' ) {
                wp_enqueue_script( 'transposh-language-toolbox-js-shortcodes', plugin_dir_url( __FILE__ ) . '../assets/js/tlt-shortcodes-min.js', array(), '1.0', true );
            }

            // Custom Languages JS
            if ( isset( $_GET['tab'] ) && $_GET['tab'] === 'custom_languages' ) {            
                wp_enqueue_script( 'transposh-language-toolbox-js-custom-languages', plugin_dir_url( __FILE__ ) . '../assets/js/tlt-custom-languages-min.js', array(), '1.0', true );
                wp_localize_script( 'transposh-language-toolbox-js-custom-languages', 'tltCustomLanguagesData', array(
                    'deleteText' => esc_html__( 'Delete', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'editText'   => esc_html__( 'Modify', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'saveText'   => esc_html__( 'Update', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'alertText1'   => esc_html__( 'Please update the language row(s) before submitting the form.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'alertText2'   => esc_html__( 'Please fill in all the fields before saving.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'alertText3'   => esc_html__( 'Please fill in all the fields before submitting the form.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'alertText4'   => esc_html__( 'Separate the data in Value using commas.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'alertText5'   => esc_html__( 'Invalid language code. Language code format should be lowercase letters, 2 or 3 characters, optionally followed by subtags or extended language ranges, separated by hyphens.', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'keyText'   => esc_html__( 'Key:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                    'valueText'   => esc_html__( 'Value:', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                ) );
            }

            // Advanced Settings JS
            if ( isset( $_GET['tab'] ) && $_GET['tab'] === 'advanced_settings' ) {  
                wp_enqueue_script( 'transposh-language-toolbox-js-advanced-settings', plugin_dir_url( __FILE__ ) . '../assets/js/tlt-advanced-settings-min.js', array(), '1.0', true );
            }
        }

        // Enqueue custom JavaScript for plugin information retrieval
        if ( isset( $_GET['tab'] ) && $_GET['tab'] === 'plugin-information' && isset( $_GET['plugin'] ) && $_GET['plugin'] === 'transposh-language-toolbox' ) {
            wp_enqueue_script( 'plugin-info-custom', plugin_dir_url( __FILE__ ) . '../assets/js/tlt-plugin-info-custom-min.js', array(), '1.0', true );
            // Enqueue Thickbox only if it's not already loaded (used in Omahoung_Update_Checker class)
            if ( ! wp_script_is( 'thickbox' ) ) {
                wp_enqueue_script( 'thickbox' );
            }
        }
    }

    /**
     * Add a submenu page under the Transposh menu.
     */
    public function add_submenu_page() {
        $allowed_roles = $this->advanced_settings->get_admin_edit_roles();

        // Add administrators to the array
        array_push( $allowed_roles, 'administrator', 'super_admin' );

        $current_user = wp_get_current_user();
        $user_roles = $current_user->roles;

        // Check if any of the user's roles are in the allowed roles array
        $intersect = array_intersect( $user_roles, $allowed_roles );

        if ( !empty( $intersect ) ) {
            add_submenu_page(
                'tp_main',
                esc_html__( 'Language Toolbox', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                esc_html__( 'Language Toolbox', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
                //'manage_options',
                'edit_posts',
                'transposh-language-toolbox',
                array( $this, 'render_submenu_page' )
            );
        }
    }

    /**
     * Add the settings link to the plugins page
     *
     * @param array $links Existing links for the plugin
     * @return array Modified links with the settings link added
     */
    public function settings_link( $links ) {
        $settings_link = '<a href="admin.php?page=transposh-language-toolbox">' . esc_html__( "Settings", TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
        array_unshift( $links, $settings_link );
        return $links;
    }

    /**
     * Register the API settings for advanced settings tab
     */
    function advanced_settings_init() {
        $this->advanced_settings->init();
    }

    /**
     * Render the submenu page content.
     */
    public function render_submenu_page() {
        $active_tab = isset( $_GET['tab'] ) ? sanitize_text_field( $_GET['tab'] ) : 'edit_translations';

        echo '<div class="wrap">';
        echo '<h1>' . esc_html__( 'Transposh Language Toolbox', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</h1>';

        // Output the tabs
        echo '<h2 class="nav-tab-wrapper">';
        echo '<a href="' . admin_url( 'admin.php?page=transposh-language-toolbox&tab=edit_translations' ) . '" class="nav-tab ' . ( $active_tab === 'edit_translations' ? 'nav-tab-active' : '' ) . '">' . esc_html__( 'Edit Translations', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
        if ( current_user_can( 'manage_options' ) ) {
            echo '<a href="' . admin_url( 'admin.php?page=transposh-language-toolbox&tab=language_shortcodes' ) . '" class="nav-tab ' . ( $active_tab === 'language_shortcodes' ? 'nav-tab-active' : '' ) . '">' . esc_html__( 'Language Shortcodes', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
            echo '<a href="' . admin_url( 'admin.php?page=transposh-language-toolbox&tab=custom_languages' ) . '" class="nav-tab ' . ( $active_tab === 'custom_languages' ? 'nav-tab-active' : '' ) . '">' . esc_html__( 'Custom Languages', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
            echo '<a href="' . admin_url( 'admin.php?page=transposh-language-toolbox&tab=advanced_settings' ) . '" class="nav-tab ' . ( $active_tab === 'advanced_settings' ? 'nav-tab-active' : '' ) . '">' . esc_html__( 'Advanced Settings', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
        }
        echo '<a href="' . admin_url( 'admin.php?page=transposh-language-toolbox&tab=about' ) . '" class="nav-tab ' . ( $active_tab === 'about' ? 'nav-tab-active' : '' ) . '">' . esc_html__( 'About', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
        echo '</h2><br>';

        $this->translation_editor = new Translation_Editor();
        $this->custom_languages = new Custom_Languages();

        // Output the content based on the active tab
        if ( $active_tab === 'edit_translations' ) {
            $this->translation_editor->render_translations_tab();
        } elseif ( $active_tab === 'language_shortcodes' ) {
            $this->shortcode->render_shortcode_explanation();
        } elseif ( $active_tab === 'custom_languages' ) {
            $this->custom_languages->render_custom_languages();
        } elseif ( $active_tab === 'advanced_settings' ) {
            $this->advanced_settings->render_form();
        } elseif ( $active_tab === 'about' ) {
            $this->about->init();
        }
        echo '</div>';
    }

    /**
     * Add screen options for the submenu page.
     */
    public function add_screen_options() {
        $active_tab = isset( $_GET['tab'] ) ? sanitize_text_field( $_GET['tab'] ) : 'edit_translations';

        if ( $active_tab !== 'edit_translations' ) {
            // Return early and don't add screen options for the "Shortcodes" tab
            return;
        }
        
        $args = array(
            'label'   => esc_html__( 'Items per page', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ),
            'default' => TRANSPOSH_EDITOR_PER_PAGE_DEFAULT,
            'option'  => TRANSPOSH_EDITOR_PER_PAGE_OPTION_NAME,
        );

        add_screen_option( 'per_page', $args );

        $this->translation_editor = new Translation_Editor();

        //Save the screen option value
        if ( isset( $_POST['wp_screen_options']['option'] ) && $_POST['wp_screen_options']['option'] === TRANSPOSH_EDITOR_PER_PAGE_OPTION_NAME ) {
            $value = (int) $_POST['wp_screen_options']['value'];
            update_user_meta( get_current_user_id(), TRANSPOSH_EDITOR_PER_PAGE_OPTION_NAME, $value );
        }
    }
}