<?php
/**
 * Update Checker Class
 *
 * @package Transposh_Language_Toolbox
 */

namespace Transposh_Language_Toolbox;

class Omahoung_Update_Checker {

    public $plugin_slug;
    public $version;
    public $cache_key;
    public $cache_allowed;

    /**
     * Constructor.
     */
    public function __construct() {

        $this->plugin_slug = plugin_basename( plugin_dir_path( __DIR__, 1 ) );
        $this->cache_key = 'transposh_language_toolbox_update';
        $this->cache_allowed = true;

        if ( is_admin() ) {
            if( ! function_exists( 'get_plugin_data' ) ) {
                require_once( ABSPATH . 'wp-admin/includes/plugin.php' );
            }
            $plugin_dir = plugin_dir_path( dirname( __FILE__, 1 ) );
            $plugin_file = $plugin_dir . $this->plugin_slug . '.php';
            $plugin_data = get_file_data( $plugin_file, array( 'Version' => 'Version' ) );
            $this->version = strval( $plugin_data['Version'] );
        
            add_filter( 'plugins_api', array( $this, 'info' ), 20, 3 );
            add_filter( 'site_transient_update_plugins', array( $this, 'update' ), 30 );
            add_action( 'upgrader_process_complete', array( $this, 'purge' ), 10, 2 );
            add_filter( 'http_request_args', array( $this, 'check_http_request_args' ), 10, 2 );
            add_filter( 'plugin_row_meta', array( $this, 'custom_plugin_meta_links' ), 10, 2 );
            
            // Add action to handle manual update check
            add_action( 'admin_init', array( $this, 'handle_manual_update_check' ) );
        }
    }

    /**
     * Add custom links to the plugin's meta row.
     */
    function custom_plugin_meta_links( $meta_links, $file ) {
        $plugin_path = $this->plugin_slug . '/' . $this->plugin_slug . '.php';
        if ( $plugin_path === $file ) {
            $found_view_details = false; // Initialize a flag
            foreach ( $meta_links as $link ) {
                if ( strpos( $link, 'plugin-install.php?tab=plugin-information' ) !== false ) {
                    $found_view_details = true;
                    break; // Stop checking once found
                }
            }
            // Only add the "View details" link if it's not already present
            if ( ! $found_view_details ) {
                $plugin_details_url = admin_url( 'plugin-install.php?tab=plugin-information&plugin=' . $this->plugin_slug . '&TB_iframe=true&width=772&height=905' );
                $meta_links[] = '<a href="' . esc_url( $plugin_details_url ) . '" class="thickbox open-plugin-details-modal">' . esc_html__( 'View details', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
            }
            // Add manual update check link
            $meta_links[] = '<a href="' . esc_url( add_query_arg( 'transposh_manual_update_check', 'true' ) ) . '">' . esc_html__( 'Check for updates', TRANSPOSH_TOOLBOX_TEXT_DOMAIN ) . '</a>';
        }
        return $meta_links;
    }

    /**
     * Perform the remote request.
     *
     * @return mixed|bool Remote response if successful, false otherwise.
     */
    public function request(){

        $remote = get_transient( $this->cache_key );

        if( false === $remote || ! $this->cache_allowed ) {

            $remote = wp_remote_get(
                TRANSPOSH_TOOLBOX_PLUGIN_REPOSITORY_URL . 'info.php',
                array(
                    'timeout' => 10,
                    'headers' => array(
                        'Accept' => 'application/json'
                    )
                )
            );

            if(
                is_wp_error( $remote )
                || 200 !== wp_remote_retrieve_response_code( $remote )
                || empty( wp_remote_retrieve_body( $remote ) )
            ) {
                return false;
            }

            set_transient( $this->cache_key, $remote, DAY_IN_SECONDS );
        }

        $remote = json_decode( wp_remote_retrieve_body( $remote ) );

        return $remote;
    }

    /**
     * Retrieve plugin information for the update API.
     *
     * @param mixed $res    Default response.
     * @param string $action Action being performed.
     * @param object $args   Additional arguments.
     * @return mixed Plugin information if applicable, false otherwise.
     */
    public function info( $res, $action, $args ) {
        /*
        * $res empty at this step
        * $action 'plugin_information'
        * $args stdClass Object ( [slug] => woocommerce [is_ssl] => [fields] => Array ( [banners] => 1 [reviews] => 1 [downloaded] => [active_installs] => 1 ) [per_page] => 24 [locale] => en_US )
        */

        // do nothing if we are not getting plugin information right now
        if( 'plugin_information' !== $action ) {
            return false;
        }

        // do nothing if it is not our plugin
        if( $this->plugin_slug !== $args->slug ) {
            return false;
        }

        // get updates
        $remote = $this->request();

        if( ! $remote ) {
            return false;
        }

        $res = new \stdClass();

        $res->name = $remote->name;
        $res->slug = $remote->slug;
        $res->version = $remote->version;
        $res->tested = $remote->tested;
        $res->requires = $remote->requires;
        $res->author = $remote->author;
        $res->author_profile = $remote->author_profile;
        $res->download_link = $remote->download_url;
        $res->trunk = $remote->download_url;
        $res->requires_php = $remote->requires_php;
        $res->last_updated = $remote->last_updated;

        $res->sections = [
            'description' => $remote->sections->description,
            'installation' => $remote->sections->installation,
            'changelog' => $remote->sections->changelog
        ];

        if( ! empty( $remote->banners ) ) {
            $res->banners = array(
                'low' => $remote->banners->low,
                'high' => $remote->banners->high
            );
        }

        return $res;
    }

    /**
     * Check for plugin updates.
     *
     * @param object $transient Transient object containing update information.
     * @return object Transient object with updated plugin information.
     */
    public function update( $transient ) {
        if ( empty( $transient->checked ) ) {
            return $transient;
        }

        $remote = $this->request();
        if(
            $remote
            && version_compare( $this->version, $remote->version, '<' )
            && version_compare( $remote->requires, get_bloginfo( 'version' ), '<' )
            && version_compare( $remote->requires_php, PHP_VERSION, '<' )
        ) {
            if ( isset( $_GET['transposh_manual_update_check'] ) && $_GET['transposh_manual_update_check'] === 'true' ) { 
                add_action( 'admin_notices', array( $this, 'display_update_available_notice' ) );
            }
            $res = new \stdClass();
            $res->slug = $this->plugin_slug;
            $res->plugin = $this->plugin_slug . '/' . $this->plugin_slug .'.php';
            $res->new_version = $remote->version;
            $res->tested = $remote->tested;
            $res->package = $remote->download_url;

			if ( ! empty( $remote->icons ) ) {
				// Access the icon URLs using object syntax
				$res->icons = array(
					'1x' => $remote->icons->{'1x'},
					'2x' => $remote->icons->{'2x'},
				);
			}

            $transient->response[ $res->plugin ] = $res;
        } else {
            if ( isset( $_GET['transposh_manual_update_check'] ) && $_GET['transposh_manual_update_check'] === 'true' ) {
                // Set an info notice if no update is available
                add_action( 'admin_notices', array( $this, 'display_no_update_notice' ) );
            }
        }

        return $transient;
    }

    /**
     * Handle manual update check.
     */
    public function handle_manual_update_check() {
        // Check if the user clicked on the manual update check link
        if ( isset( $_GET['transposh_manual_update_check'] ) && $_GET['transposh_manual_update_check'] === 'true' ) {
            // Delete the transient to force a new update check
            delete_transient( $this->cache_key );
        }
    }

    /**
     * Display notice for available update.
     */
    public function display_update_available_notice() {
        echo '<div class="notice notice-success is-dismissible">';
        echo '<p>' . esc_html__( 'A new update is available for Transposh Language Toolbox!', 'transposh-language-toolbox' ) . '</p>';
        echo '</div>';
    }

    /**
     * Display notice for no available update.
     */
    public function display_no_update_notice() {
        echo '<div class="notice notice-info is-dismissible">';
        echo '<p>' . esc_html__( 'Transposh Language Toolbox is up to date.', 'transposh-language-toolbox' ) . '</p>';
        echo '</div>';
    }

    /**
     * Purge the cache when a plugin update is completed.
     *
     * @param string $upgrader_object Upgrader object.
     * @param array $options          Update options.
     */
    public function purge( $upgrader_object, $options ) {

        if (
            $this->cache_allowed
            && 'update' === $options['action']
            && 'plugin' === $options[ 'type' ]
        ) {
            // just clean the cache when new plugin version is installed
            delete_transient( $this->cache_key );
        }
    }

    /**
     * Filter the HTTP request arguments before making an HTTP request.
     * Send "update" action to plugin repository
     *
     * @param array  $args The original HTTP request arguments.
     * @param string $url  The request URL.
     *
     * @return array The modified HTTP request arguments.
     */
    public function check_http_request_args( $args, $url ) {
        // URL to the plugin's ZIP file repository
        $plugin_zip_repository_url = TRANSPOSH_TOOLBOX_PLUGIN_REPOSITORY_URL . 'transposh-language-toolbox.zip';
    
        if ( $url === $plugin_zip_repository_url ) {
            // Get the WordPress version
            global $wp_version;

            // Get the domain
            $domain = home_url();

            // Get the PHP version
            $php_version = phpversion();

            // Prepare the data to be sent to the server
            $data = array(
                'action' => 'update',
                'plugin_name' => $this->plugin_slug,
                'version' => $this->version,
                'domain' => $domain,
                'transposh_version' => TRANSPOSH_TOOLBOX_TRANSPOSH_VERSION,
                'wp_version'    => $wp_version,
                'php_version'   => $php_version,
            );
    
            // Send the data to repository using a POST request
            $response = wp_remote_post( TRANSPOSH_TOOLBOX_PLUGIN_REPOSITORY_URL . 'plugin-logger.php', array(
                'method' => 'POST',
                'body'   => $data,
            ) );
    
            // Can check the response if needed
            if ( is_wp_error( $response ) ) {
                // Handle the error if the request fails
            }
        }
    
        return $args;
    }
}